@@ -32,13 +32,13 @@
 #
 # Images which we use with our chat balloons.
 #
-image create photo CHAT_notes -file "$MV(icondir)/notes.gif"
-image create photo CHAT_love -file "$MV(icondir)/hearts.gif"
-image create photo CHAT_steam -file "$MV(icondir)/steam.gif"
-image create photo CHAT_idea -file "$MV(icondir)/bulb.gif"
-image create photo CHAT_smile -file "$MV(icondir)/smile.gif"
-image create photo CHAT_frown -file "$MV(icondir)/frown.gif"
-image create photo CHAT_wink -file "$MV(icondir)/wink.gif"
+image create photo OpenVerse_Image_CHAT_notes -file "$MV(icondir)/notes.gif"
+image create photo OpenVerse_Image_CHAT_love -file "$MV(icondir)/hearts.gif"
+image create photo OpenVerse_Image_CHAT_steam -file "$MV(icondir)/steam.gif"
+image create photo OpenVerse_Image_CHAT_idea -file "$MV(icondir)/bulb.gif"
+image create photo OpenVerse_Image_CHAT_smile -file "$MV(icondir)/smile.gif"
+image create photo OpenVerse_Image_CHAT_frown -file "$MV(icondir)/frown.gif"
+image create photo OpenVerse_Image_CHAT_wink -file "$MV(icondir)/wink.gif"
 
 #
 # Will fill the names button with people who are in the channel.
@@ -180,7 +180,7 @@ proc StopLog {} {
 proc NickComplete {} {
 	global MV
 
-	DebugIt "-- In NickComplete"
+	DebugIt "-- In NickComplete" other
         set stuff [$MV(chat_entry) get]
 	if {![string compare $stuff ""]} {return}
 	set wrds [split $stuff " "]
@@ -199,18 +199,18 @@ proc SetRoom {what size} {
 
 	if {[file exists "$MV(roomdir)/$what"]} {
 		if {[file size "$MV(roomdir)/$what"] != $size} {
-			DebugIt "size mismatch for $what"
+			DebugIt "size mismatch for $what" other
 			catch {file delete "$MV(roomdir)/$what"}
 			if {[lsearch -exact $MV(roomloads) $what] == -1} {
-				image create photo room -file "$MV(homedir)/icons/loading.gif"
+				image create photo OpenVerse_Image_room -file "$MV(homedir)/icons/loading.gif"
 				SendToServer "DCCSENDROOM $what"
 			}
 		} else {
-			image create photo room -file "$MV(roomdir)/$what"
+			image create photo OpenVerse_Image_room -file "$MV(roomdir)/$what"
 		}
 	} else {
 		if {[lsearch -exact $MV(roomloads) $what] == -1} {
-			image create photo room -file "$MV(homedir)/icons/loading.gif"
+			image create photo OpenVerse_Image_room -file "$MV(homedir)/icons/loading.gif"
 			SendToServer "DCCSENDROOM $what"
 		}
 	}
@@ -229,6 +229,43 @@ proc WHOISUser {who hostmask} {
 	ProcChat $who "My host information is $hostmask" 0 3 $MV(colors.whois.baloon)
 }
 
+proc GetChatRows {text} {
+	global MV
+	set words [split $text]
+	set word_ptr 0
+	set last_word [llength $words]
+	if {$last_word == 1} {return 1}
+	set rows 1
+	set line ""
+	set last_word_ptr -1
+	while {$word_ptr <= $last_word} {
+DebugIt "word_ptr is $word_ptr" other
+update idletasks
+		append line [lindex $words $word_ptr]
+		set width [font measure "$MV(font.balloon.$MV(font.balloon).$MV(font.balloon.style)) -size $MV(font.balloon.size)" $line]
+		if {$width >= 240} {
+DebugIt "puts us over the top." other
+			incr rows
+			set incremented 1
+			set line ""
+			if {$word_ptr == $last_word_ptr} {
+				incr word_ptr
+			} else {
+				set last_word_ptr $word_ptr
+			}
+		} else {
+DebugIt "Still within range" other
+			set incremented 0
+			incr word_ptr
+		}
+	}
+	if $incremented {
+		return $rows
+	} else {
+		return [expr $rows + 1]
+	}
+}
+
 proc ProcChat {who what que system baloon} {
 	global MV
 
@@ -266,7 +303,7 @@ proc ProcChat {who what que system baloon} {
 	#	7	Idea Chat
 	#	8	Smiling Chat
 	#	9	Frowning Chat
-	#	10	Winking Chat
+	#	A	Winking Chat
 	#
 	switch $system {
 		0 {set textcolor $MV(colors.chat.text)}
@@ -279,7 +316,7 @@ proc ProcChat {who what que system baloon} {
 		7 {set textcolor $MV(colors.chat.text)}
 		8 {set textcolor $MV(colors.chat.text)}
 		9 {set textcolor $MV(colors.chat.text)}
-		10 {set textcolor $MV(colors.chat.text)}
+		A {set textcolor $MV(colors.chat.text)}
 		default {set textcolor "black"}
 	}
 	set oldwhat $what
@@ -293,25 +330,22 @@ proc ProcChat {who what que system baloon} {
 		}
 		set text "$string$text"
 	}
-	set what $text	
-	if {[string length $what] >= 40} {
-		set width 240
-		set rows [expr [string length $what] / 40]
+	set what $text
+	set rows [GetChatRows $text]
+	DebugIt "Rows is set to $rows" other
+	set font_height [lindex [font metrics "$MV(font.balloon.$MV(font.balloon).$MV(font.balloon.style)) -size $MV(font.balloon.size)"] 5]
+	set fwidth [font measure "$MV(font.balloon.$MV(font.balloon).$MV(font.balloon.style)) -size $MV(font.balloon.size)" $text]
+	if {$fwidth > 260} {
+		set width 260
+		if {$rows == 1} {set rows [expr int($fwidth / 260) + 1]}
 		if {$rows < 3} {set rows 2}
-		set height [expr ([string length $what] / 40) * (25 - ($rows * 2))]
-		set hoff [expr $rows * 7]
+		set height [expr $rows * $font_height]
+		set hoff [expr int($rows * 1.5)]
 	} else {
-		if {[string length $what] > 3} {
-			set width [expr [string length $what] * 9]
-			set height 30
-			set rows 2
-			set hoff [expr [string length $what] / 2]
-		} else {
-			set width [expr [string length $what] * 40]
-			set height 30
-			set rows 2
-			set hoff 0
-		}
+		set width [expr $fwidth + 15]
+		set height 30
+		set rows 2
+		set hoff 2
 	}
 	if {$system >= 4} {incr width 30}
 	if {$width > 300} {set width 300}
@@ -324,21 +358,25 @@ proc ProcChat {who what que system baloon} {
 		set yoff 1
 	}
 	set MV($who.baloon) [.top.c create polygon \
-	[expr $MV($who.x) + ($xoff - (20 * $yoff))] [expr $MV($who.y) + $MV($who.baloon_y)] \
-	[expr $MV($who.x) + $xoff] [expr $MV($who.y) + $MV($who.baloon_y)] \
-	[expr $MV($who.x) + $xoff + ($yoff * 20)] [expr ($MV($who.y)+ $MV($who.baloon_y)) - $hoff] \
-	[expr $MV($who.x) + $xoff + ($width * $yoff) - ($yoff * 20)] [expr ($MV($who.y)+ $MV($who.baloon_y)) - $hoff] \
-	[expr $MV($who.x) + $xoff + ($width * $yoff)] [expr $MV($who.y) + $MV($who.baloon_y)] \
-	[expr $MV($who.x) + $xoff + ($width * $yoff)] [expr ($MV($who.y)+ $MV($who.baloon_y)) + $height]\
-	[expr $MV($who.x) + $xoff + ($width * $yoff) - ($yoff * 20)] [expr ($MV($who.y)+ $MV($who.baloon_y)) + $height +$hoff] \
-	[expr $MV($who.x) + $xoff + ($yoff * 20)] [expr ($MV($who.y)+ $MV($who.baloon_y)) + $hoff + $height] \
-	[expr $MV($who.x) + $xoff] [expr ($MV($who.y)+ $MV($who.baloon_y)) + $height] \
-	[expr $MV($who.x) + $xoff] [expr ($MV($who.y)+ $MV($who.baloon_y)) + 5] \
-	[expr $MV($who.x) + ($xoff - (20 * $yoff))] [expr $MV($who.y) + $MV($who.baloon_y)] \
-	-fill $baloon -outline black -smooth 1]
-	set MV($who.text) [.top.c create text [expr $MV($who.x) + $xoff + \
-		($yoff * ($width / 2))] [expr ($MV($who.y)+ $MV($who.baloon_y)) + (8 * $rows)] \
-		-width 230 -text $what -fill $textcolor]
+		[expr $MV($who.x) + ($xoff - (20 * $yoff))] [expr $MV($who.y) + $MV($who.baloon_y)] \
+		[expr $MV($who.x) + $xoff] [expr $MV($who.y) + $MV($who.baloon_y) - 5] \
+		[expr $MV($who.x) + $xoff] [expr $MV($who.y) + $MV($who.baloon_y) - ($height / 2)] \
+		[expr $MV($who.x) + $xoff + ($yoff * 10)] [expr $MV($who.y) + $MV($who.baloon_y) - ($height / 2) -  $hoff] \
+		[expr $MV($who.x) + $xoff + (($width - 10) * $yoff)] [expr $MV($who.y) + $MV($who.baloon_y) - ($height / 2) - $hoff] \
+		[expr $MV($who.x) + $xoff + ($width * $yoff)] [expr $MV($who.y) + $MV($who.baloon_y) - ($height / 2)] \
+		[expr $MV($who.x) + $xoff + ($width * $yoff)] [expr $MV($who.y) + $MV($who.baloon_y) + ($height / 2)] \
+		[expr $MV($who.x) + $xoff + (($width - 10) * $yoff)] [expr $MV($who.y) + $MV($who.baloon_y) + ($height / 2) + $hoff] \
+		[expr $MV($who.x) + $xoff + ($yoff * 10)] [expr $MV($who.y) + $MV($who.baloon_y) + ($height / 2) + $hoff] \
+		[expr $MV($who.x) + $xoff] [expr $MV($who.y) + $MV($who.baloon_y) + ($height / 2)] \
+		[expr $MV($who.x) + $xoff] [expr $MV($who.y) + $MV($who.baloon_y) + 5] \
+		[expr $MV($who.x) + ($xoff - (20 * $yoff))] [expr $MV($who.y) + $MV($who.baloon_y)] \
+		-fill $baloon -outline black -smooth 1]
+	# [expr $MV(x) + ($width / 2)] [expr $MV(y) + ($height / 2)]
+	set MV($who.text) [.top.c create text \
+		[expr $MV($who.x) + ($xoff - (20 * $yoff)) + ($yoff * ($width / 2)) + ($yoff * 20)] \
+		[expr $MV($who.y) + $MV($who.baloon_y) - ($height / 2) + ($height / 2)] \
+		-width 240 -text $what -fill $textcolor \
+		-font "$MV(font.balloon.$MV(font.balloon).$MV(font.balloon.style)) -size $MV(font.balloon.size)"]
 	set what $oldwhat
 	switch $system {
 		0 {set log 1;set SendToChat "<$who> $what\n"}
@@ -351,9 +389,9 @@ proc ProcChat {who what que system baloon} {
 			incr MV(killnotes)
 			set MV(NotesKill.$MV(killnotes)) "$who"
 			set MV($who.notes) [.top.c create image \
-				[expr $MV($who.x) + $xoff + ($yoff * 8)] \
-				[expr (($MV($who.y)+ $MV($who.baloon_y)) - $hoff) + (((($MV($who.y)+ $MV($who.baloon_y)) +$hoff + $height) - (($MV($who.y)+ $MV($who.baloon_y)) - $hoff)) / 2)] \
-				-image CHAT_notes]
+				[expr $MV($who.x) + $xoff + ($yoff * 10)] \
+				[expr $MV($who.y) + $MV($who.baloon_y)] \
+				-image OpenVerse_Image_CHAT_notes]
 			after [expr $MV(staytime) * 1000] "KillNotes $MV(killnotes)"
 		}
 		5 {
@@ -362,9 +400,9 @@ proc ProcChat {who what que system baloon} {
 			incr MV(killlove)
 			set MV(LoveKill.$MV(killlove)) "$who"
 			set MV($who.love) [.top.c create image \
-				[expr $MV($who.x) + $xoff + ($yoff * 8)] \
-				[expr (($MV($who.y)+ $MV($who.baloon_y)) - $hoff) + (((($MV($who.y)+ $MV($who.baloon_y)) +$hoff + $height) - (($MV($who.y)+ $MV($who.baloon_y)) - $hoff)) / 2)] \
-				-image CHAT_love]
+				[expr $MV($who.x) + $xoff + ($yoff * 10)] \
+				[expr $MV($who.y) + $MV($who.baloon_y)] \
+				-image OpenVerse_Image_CHAT_love]
 			after [expr $MV(staytime) * 1000] "KillLove $MV(killlove)"
 		}
 		6 {
@@ -373,9 +411,9 @@ proc ProcChat {who what que system baloon} {
 			incr MV(killsteam)
 			set MV(SteamKill.$MV(killsteam)) "$who"
 			set MV($who.steam) [.top.c create image \
-				[expr $MV($who.x) + $xoff + ($yoff * 8)] \
-				[expr (($MV($who.y)+ $MV($who.baloon_y)) - $hoff) + (((($MV($who.y)+ $MV($who.baloon_y)) +$hoff + $height) - (($MV($who.y)+ $MV($who.baloon_y)) - $hoff)) / 2)] \
-				-image CHAT_steam]
+				[expr $MV($who.x) + $xoff + ($yoff * 10)] \
+				[expr $MV($who.y) + $MV($who.baloon_y)] \
+				-image OpenVerse_Image_CHAT_steam]
 			after [expr $MV(staytime) * 1000] "KillSteam $MV(killsteam)"
 		}
 		7 {
@@ -384,9 +422,9 @@ proc ProcChat {who what que system baloon} {
 			incr MV(killidea)
 			set MV(IdeaKill.$MV(killidea)) "$who"
 			set MV($who.idea) [.top.c create image \
-				[expr $MV($who.x) + $xoff + ($yoff * 8)] \
-				[expr (($MV($who.y)+ $MV($who.baloon_y)) - $hoff) + (((($MV($who.y)+ $MV($who.baloon_y)) +$hoff + $height) - (($MV($who.y)+ $MV($who.baloon_y)) - $hoff)) / 2)] \
-				-image CHAT_idea]
+				[expr $MV($who.x) + $xoff + ($yoff * 10)] \
+				[expr $MV($who.y) + $MV($who.baloon_y)] \
+				-image OpenVerse_Image_CHAT_idea]
 			after [expr $MV(staytime) * 1000] "KillIdea $MV(killidea)"
 		}
 		8 {
@@ -395,9 +433,9 @@ proc ProcChat {who what que system baloon} {
 			incr MV(killsmile)
 			set MV(SmileKill.$MV(killsmile)) "$who"
 			set MV($who.smile) [.top.c create image \
-				[expr $MV($who.x) + $xoff + ($yoff * 12)] \
-				[expr (($MV($who.y)+ $MV($who.baloon_y)) - $hoff) + (((($MV($who.y)+ $MV($who.baloon_y)) +$hoff + $height) - (($MV($who.y)+ $MV($who.baloon_y)) - $hoff)) / 2)] \
-				-image CHAT_smile]
+				[expr $MV($who.x) + $xoff + ($yoff * 10)] \
+				[expr $MV($who.y) + $MV($who.baloon_y)] \
+				-image OpenVerse_Image_CHAT_smile]
 			after [expr $MV(staytime) * 1000] "KillSmile $MV(killsmile)"
 		}
 		9 {
@@ -406,20 +444,20 @@ proc ProcChat {who what que system baloon} {
 			incr MV(killfrown)
 			set MV(FrownKill.$MV(killfrown)) "$who"
 			set MV($who.frown) [.top.c create image \
-				[expr $MV($who.x) + $xoff + ($yoff * 12)] \
-				[expr (($MV($who.y)+ $MV($who.baloon_y)) - $hoff) + (((($MV($who.y)+ $MV($who.baloon_y)) +$hoff + $height) - (($MV($who.y)+ $MV($who.baloon_y)) - $hoff)) / 2)] \
-				-image CHAT_frown]
+				[expr $MV($who.x) + $xoff + ($yoff * 10)] \
+				[expr $MV($who.y) + $MV($who.baloon_y)] \
+				-image OpenVerse_Image_CHAT_frown]
 			after [expr $MV(staytime) * 1000] "KillFrown $MV(killfrown)"
 		}
-		10 {
+		A {
 			set log 1
 			set SendToChat "<;)>$who<(;> $what\n"
 			incr MV(killwink)
 			set MV(WinkKill.$MV(killwink)) "$who"
 			set MV($who.wink) [.top.c create image \
-				[expr $MV($who.x) + $xoff + ($yoff * 12)] \
-				[expr (($MV($who.y)+ $MV($who.baloon_y)) - $hoff) + (((($MV($who.y)+ $MV($who.baloon_y)) +$hoff + $height) - (($MV($who.y)+ $MV($who.baloon_y)) - $hoff)) / 2)] \
-				-image CHAT_wink]
+				[expr $MV($who.x) + $xoff + ($yoff * 10)] \
+				[expr $MV($who.y) + $MV($who.baloon_y)] \
+				-image OpenVerse_Image_CHAT_wink]
 			after [expr $MV(staytime) * 1000] "KillWink $MV(killwink)"
 		}
 		default {set log 1;set SendToChat "<$who> $what\n"}
@@ -440,71 +478,110 @@ proc ProcChat {who what que system baloon} {
 proc KillNotes {which} {
 	global MV
 
-	DebugIt "Killing Music Notes!"
+	DebugIt "Killing Music Notes!" other
 	set who "$MV(NotesKill.$which)"
-	unset MV(NotesKill.$which)
 	.top.c delete $MV($who.notes)
+	#
+	# Clean up the memory which this user was using.
+	#
+	catch {unset MV(NotesKill.$which)}
+	catch {unset MV($who.notes)}
 }
 
 proc KillLove {which} {
 	global MV
 
-	DebugIt "Killing Little Hearts (poor things)!"
+	DebugIt "Killing Little Hearts (poor things)!" other
 	set who "$MV(LoveKill.$which)"
-	unset MV(LoveKill.$which)
 	.top.c delete $MV($who.love)
+	#
+	# Clean up the memory which this user was using.
+	#
+	catch {unset MV(LoveKill.$which)}
+	catch {unset MV($who.love)}
 }
 
 proc KillSteam {which} {
 	global MV
 
-	DebugIt "Killing Steam!"
+	DebugIt "Killing Steam!" other
 	set who "$MV(SteamKill.$which)"
-	unset MV(SteamKill.$which)
 	.top.c delete $MV($who.steam)
+	#
+	# Clean up the memory which this user was using.
+	#
+	catch {unset MV(SteamKill.$which)}
+	catch {unset MV($who.steam)}
 }
 
 proc KillIdea {which} {
 	global MV
 
-	DebugIt "Killing Idea (they are a threat to free speach and must be silenced!!"
+	DebugIt "Killing Idea (they are a threat to free speach and must be silenced!!" other
 	set who "$MV(IdeaKill.$which)"
-	unset MV(IdeaKill.$which)
 	.top.c delete $MV($who.idea)
+	#
+	# Clean up the memory which this user was using.
+	#
+	catch {unset MV(IdeaKill.$which)}
+	catch {unset MV($who.idea)}
 }
 
 proc KillSmile {which} {
 	global MV
 
-	DebugIt "Killing Smile"
+	DebugIt "Killing Smile" other
 	set who "$MV(SmileKill.$which)"
-	unset MV(SmileKill.$which)
 	.top.c delete $MV($who.smile)
+	#
+	# Clean up the memory which this user was using.
+	#
+	catch {unset MV(SmileKill.$which)}
+	catch {unset MV($who.smile)}
 }
 proc KillFrown {which} {
 	global MV
 
-	DebugIt "Killing Frown"
+	DebugIt "Killing Frown" other
 	set who "$MV(FrownKill.$which)"
-	unset MV(FrownKill.$which)
 	.top.c delete $MV($who.frown)
+	#
+	# Clean up the memory which this user was using.
+	#
+	catch {unset MV(FrownKill.$which)}
+	catch {unset MV($who.frowm)}
 }
 proc KillWink {which} {
 	global MV
 
-	DebugIt "Killing Wink"
+	DebugIt "Killing Wink" other
 	set who "$MV(WinkKill.$which)"
-	unset MV(WinkKill.$which)
 	.top.c delete $MV($who.wink)
+	#
+	# Clean up the memory which this user was using.
+	#
+	catch {unset MV(WinkKill.$which)}
+	catch {unset MV($who.wink)}
 }
 
 proc KillBaloon {which} {
 	global MV
 
 	set who "$MV(BaloonKill.$which)"
-	unset MV(BaloonKill.$which)
+	if {[lsearch -exact $MV(people) $who] == -1 && $who != $MV(nick)} {
+		DebugIt "KillBaloon: Called KillBaloon for user which is no longer logged on." other
+		set MV($who.chatque) {}
+		set MV($who.haschat) 0
+	}
 	.top.c delete $MV($who.text)
 	.top.c delete $MV($who.baloon)
+	#
+	# Clean up the memory this was using.
+	#
+	catch {unset MV($who.text)}
+	catch {unset MV($who.baloon)}
+	catch {unset MV(BaloonKill.$which)}
+
 	if {[llength $MV($who.chatque)]} {
 		set stuff [lindex $MV($who.chatque) 0]
 		set system [string range $stuff 0 0]
@@ -514,7 +591,12 @@ proc KillBaloon {which} {
 		set MV($who.chatque) [lreplace $MV($who.chatque) 0 0]
 		ProcChat $who $what 1 $system $color
 	} else {
-		set MV($who.haschat) 0
+		if {[lsearch -exact $MV(people) $who] != -1 || $who == $MV(nick)} {
+			set MV($who.haschat) 0
+		} else {
+			catch {unset MV($who.haschat)}
+			catch {unset MV($who.chatque)}
+		}
 	}
 }
 
@@ -522,7 +604,16 @@ proc NewPerson {who x y image xx yy size bx by} {
 	global MV
 
 	if {![string compare $who $MV(nick)]} {return}
-	if {[lsearch -exact $MV(people) $who] != -1} {return}
+	if {[lsearch -exact $MV(people) $who] != -1} {
+		DebugIt "New User $who is already logged on" other
+		return
+	}
+	if {[TestPosNum $x] || [TestPosNum $y] || [TestNum $xx] || \
+	[TestNum $yy] || [TestPosNum $size] || [TestNum $bx] || \
+	[TestNum $by]} {
+		DebugIt "New User $who has invalid numerics (ignoring)" other
+		return
+	}
 	lappend MV(people) $who
 	set MV($who.name) $who
 	set MV($who.downloading) 0
@@ -548,17 +639,17 @@ proc NewPerson {who x y image xx yy size bx by} {
 	
 	InsertIntoChat 1 "<*>$who<*> Entered the room\n"
 	if {![string compare $image "default.gif"]} {
-		image create photo $who -file "$MV(images)/default.gif"
-		set MV($who.icon) [.top.c create image $x $y -image $who]
+		image create photo OpenVerse_User_Image_$who -file "$MV(images)/default.gif"
+		set MV($who.icon) [.top.c create image $x $y -image OpenVerse_User_Image_$who]
 		if $MV(names) {ShowName $who}
 		DoNames
 		return
 	}
 	if ![file exists "$MV(rem_images)/$image"] {
-		image create photo $who -file "$MV(images)/default.gif"
-		set MV($who.icon) [.top.c create image $x $y -image $who]
+		image create photo OpenVerse_User_Image_$who -file "$MV(images)/default.gif"
+		set MV($who.icon) [.top.c create image $x $y -image OpenVerse_User_Image_$who]
 		if {[lsearch -exact $MV(downloads) $image] == -1} {
-			DebugIt "I do not have $image"
+			DebugIt "I do not have $image" other
 			if $MV(download_avatars) {
 				SendToServer "DCCSENDAV $image"
 				set MV($who.downloading) 1
@@ -566,21 +657,21 @@ proc NewPerson {who x y image xx yy size bx by} {
 		}
 	} else {
 		if {[file size "$MV(rem_images)/$image"] != $size} {
-			image create photo $who -file "$MV(images)/default.gif"
-			set MV($who.icon) [.top.c create image $x $y -image $who]
+			image create photo OpenVerse_User_Image_$who -file "$MV(images)/default.gif"
+			set MV($who.icon) [.top.c create image $x $y -image OpenVerse_User_Image_$who]
 			if {[lsearch -exact $MV(downloads) $image] == -1} {
-				DebugIt "size mismatch for $image"
+				DebugIt "size mismatch for $image" other
 				if $MV(download_avatars) {
 					SendToServer "DCCSENDAV $image"
 					set MV($who.downloading) 1
 				}
 			}
 		} else {
-			catch {image create photo $who -file "$MV(rem_images)/$image"} err
-			if {[string compare $err $who]} {
-				image create photo $who -file "$MV(images)/default.gif"
+			catch {image create photo OpenVerse_User_Image_$who -file "$MV(rem_images)/$image"} err
+			if {[string compare $err "OpenVerse_User_Image_$who"]} {
+				image create photo OpenVerse_User_Image_$who -file "$MV(images)/default.gif"
 			}
-			set MV($who.icon) [.top.c create image $x $y -image $who]
+			set MV($who.icon) [.top.c create image $x $y -image OpenVerse_User_Image_$who]
 		}
 	}
 	if $MV(names) {
@@ -632,15 +723,38 @@ proc PersonLeft {who} {
 	set MV($who.nomoremove) 1
 	.top.c delete $MV($who.icon)
 	InsertIntoChat 1 "<*>$who<*> Left the room\n"
-	if {$MV($who.haschat)} {
-		.top.c delete $MV($who.text)
-		.top.c delete $MV($who.baloon)
-		set MV($who.chatque) {}
-		set MV(haschat) 0
-	}
 	if $MV(names) {
 		KillName $who
 	}
+	# 
+	# Clean up the memory this user was using!
+	#
+	image delete OpenVerse_User_Image_$who
+	catch {unset MV($who.name)}  
+	catch {unset MV($who.avatar)}
+	catch {unset MV($who.x)}
+	catch {unset MV($who.y)}
+	catch {unset MV($who.lastsaid)}
+	catch {unset MV($who.name_x_offset)}
+	catch {unset MV($who.name_y_offset)}
+	catch {unset MV($who.baloon_x)}
+	catch {unset MV($who.baloon_y)}
+	catch {unset MV($who.nomoremove)}
+	catch {unset MV($who.downloading)}
+	catch {unset MV($who.icon)}  
+	catch {unset MV($who.moves)}
+	catch {unset MV($who.moving)}
+	catch {unset MV($who.name_br_x)}
+	catch {unset MV($who.name_br_y)}
+	catch {unset MV($who.name_tl_x)}
+	catch {unset MV($who.name_tl_y)}
+	catch {unset MV($who.nameplate)}
+	catch {unset MV($who.nametext)}
+	catch {unset MV($who.chattime)}   
+	catch {unset MV($who.current_x)}   
+	catch {unset MV($who.current_y)}   
+	catch {unset MV($who.haschat)}
+	catch {unset MV($who.chatque)}
 	DoNames
 }
 
@@ -800,7 +914,7 @@ proc SendPrivMsg {which} {
 #
 # Test a variable to see if it is a number. It will return 0 if it passes
 # the test, it will return 1 if it fails the test. Negative numbers are
-# considered.
+# considered to be valid numbers in this test.
 #
 proc TestNum {number} {
         if {[string length [string trim $number -0123456789]]} {
@@ -810,6 +924,19 @@ proc TestNum {number} {
         }
 }
 
+#
+# Test a variable to see if it is a number. It will return 0 if it passes
+# the test, it will return 1 if it fails the test. Negative numbers are
+# considered invalid numbers in this test.
+#
+proc TestPosNum {number} {
+        if {[string length [string trim $number 0123456789]]} {
+                return 1
+        } else {
+                return 0                  
+        }
+}
+
 #
 # Change the user's nick and provide a place for plugin's to link to.
 #
@@ -847,7 +974,7 @@ proc IgnoreUser {who how} {
 	global MV
 	
 	set who $MV(whois.$who)
-	DebugIt "Ignoring $who ($how)"
+	DebugIt "Ignoring $who ($how)" other
 
 	switch -exact -- [string tolower $how] {
 		"all" {
@@ -895,7 +1022,7 @@ proc UnIgnoreUser {who how} {
 	global MV
 	
 	set who $MV(whois.$who)
-	DebugIt "UnIgnoring $who ($how)"
+	DebugIt "UnIgnoring $who ($how)" other
 
 	switch -exact -- [string tolower $how] {
 		"all" {
@@ -1114,7 +1241,7 @@ proc BuildPersonalInfo {who} {
 		-variable MV($who.personal.sex) -value "Female"
 	radiobutton $win.info.meat.inf.d.oc -text Other \
 		-variable MV($who.personal.sex) -value "Other"
-	DebugIt "(PersonalSex) - |$MV($who.personal.sex)|"
+	DebugIt "(PersonalSex) - |$MV($who.personal.sex)|" other
 
 	frame $win.info.meat.txt
 	text $win.info.meat.txt.t -height 10 -width 60 -wrap word
@@ -1142,3 +1269,20 @@ proc BuildPersonalInfo {who} {
 	pack $win.info.meat.txt.t -fill both -expand y
 	$win.info.meat.txt.t insert end $MV($who.personal.text)	
 }
+
+
+#
+# ThrowError
+#
+# Usage: ThrowError "stuff"
+#
+# Will pop up a dialog window with an error in it giving the user only ONE
+# button.... "OK" as an option, which will close the window.
+#
+proc ThrowError {stuff} {
+	global MV
+	incr MV(popup_window)
+	tk_dialog .openverse_error_$MV(popup_window) \
+		"An Error Has Occured!" \
+		"$stuff" "" 0 "OK"
+}
