@@ -33,15 +33,10 @@
 #
 # THIS IS THE DEFAULT SERVER AND PORT
 #
-set MV(roomhost) "world.openverse.org"
-set MV(roomport) "7000"
-
-
-#
-# TODO:
-# All of these things need to be documented.
-#
-
+set MV(default_host) "world.openverse.org";	# Setup uses this one so as not to adopt the current host as the default host each time setup is pressed.
+set MV(default_port) "7000";			# Setup uses this one so as not to adopt the current port as the default port each time setup is pressed.
+set MV(roomhost) "world.openverse.org";		# This is our default server hostname.
+set MV(roomport) "7000";			# This is our default server port.
 #
 # Significant RunTime Variables
 #
@@ -51,65 +46,73 @@ set MV(roomport) "7000"
 #
 # Directories and files
 #
-set MV(personal.image) "$MV(homedir)/icons/pinfo.gif"
-set MV(personal.homepage) "http://openverse.org/"
-set MV(bookmarks) "$MV(homedir)/passageways.txt"
-set MV(configfile) "$MV(homedir)/MetaV.cfg"
-set MV(history) "$MV(homedir)/history.txt"
-set MV(rem_images) "$MV(homedir)/rimages"
-set MV(plugindir) "$MV(homedir)/plugins"
-set MV(objects) "$MV(homedir)/objects"
-set MV(paneldir) "$MV(homedir)/panels"
-set MV(images) "$MV(homedir)/images"
-set MV(icondir) "$MV(homedir)/icons"
-set MV(roomdir) "$MV(homedir)/rooms"
-set MV(anims) "$MV(homedir)/anims"
-set MV(avatar) "default.gif"
-set MV(anim) "default.av"
+set MV(personal.image) "$MV(homedir)/icons/pinfo.gif";	# The default image to use for personal info.
+set MV(bookmarks) "$MV(homedir)/passageways.txt";	# Our bookmarks file.
+set MV(configfile) "$MV(homedir)/MetaV.cfg";		# The name of our configuration file.
+set MV(history) "$MV(homedir)/history.txt";		# The name of the file which holds our room history.
+set MV(rem_images) "$MV(homedir)/rimages";		# The dir where we keep remote  avatar images.
+set MV(plugindir) "$MV(homedir)/plugins";		# The directory where plugins are placed.
+set MV(objects) "$MV(homedir)/objects";			# Where do we store server object images?
+set MV(paneldir) "$MV(homedir)/panels";			# Where we keep our panel files.
+set MV(images) "$MV(homedir)/images";			# The dir where we keep our own avatar images.
+set MV(icondir) "$MV(homedir)/icons";			# Where do we store the application's icon files?
+set MV(roomdir) "$MV(homedir)/rooms";			# The dir where we store room images.
+set MV(anims) "$MV(homedir)/anims";			# The directory where .av files are stored.
+set MV(avatar) "default.gif";				# Your avatar's image name.
+set MV(anim) "default.av";				# Your avatar's .av file name.
+# MV(helpdir);						# The directory we keep our help files in, OpenVerse.tcl sets this platform specific variable.
+# MV(libbase);						# Used with UNIX type machines to copy files from the central install location.
+# MV(libdir);						# The directory we keep our library files in, OpenVerse.tcl sets this platform specific  variable. 
+
 #
 # Strings
 #
-set MV(personal.info) "Type Information About Yourself Here!"
-set MV(personal.email) "myemail@example.com"
-set MV(download_dir) "$MV(homedir)/download"
-set MV(roomname) "Welcome To The OpenVerse"
-set MV(personal.realname) "My Real Name"
-set MV(personal.availability) "Never"
-set MV(browser_cmd) "netscape"
-set MV(personal.sex) "Other"
-set MV(nick) "Guest"
+set MV(personal.info) "Type Information About Yourself Here!";	# Info about us as displayed in our personal information section.
+set MV(personal.homepage) "http://openverse.org/";		# Our home page as displayed in our personal info.
+set MV(personal.email) "myemail@example.com";	# Our email address as displayed in our personal info.
+set MV(download_dir) "$MV(homedir)/download";	# Where do we download files to.
+set MV(roomname) "Welcome To The OpenVerse";	# The name of the room which we are currently in.
+set MV(personal.realname) "My Real Name";	# Our real name as displayed in our personal info.
+set MV(personal.availability) "Never";		# How to make our personal information available to other users.
+set MV(font.balloon.style) "normal";		# The style of the font we use in balloons.
+set MV(browser_cmd) "netscape";			# This is the command to run our web browser from.
+set MV(personal.sex) "Other";			# The sex which is displayed in our personal Info.
+set MV(ChtButState) "show";			# Is the chat button bar shown or hidden?
+set MV(nick) "Guest";				# Your nick name!
 #
 # Numeric Variables
 #
-set MV(use_windowmanager_colors) 0
-set MV(download_avatars) 1
-set MV(name_y_offset) 40
-set MV(anim.baloon_x) 24
-set MV(sort_bookmarks) 0
-set MV(name_x_offset) 0
-set MV(anim.baloon_y) 6
-set MV(sendbuffer) 4096
-set MV(honor_effects) 1
-set MV(history_max) 10
-set MV(getbuffer) 1024
-set MV(dcctimeout) 180
-set MV(anim.y_off) 20
-set MV(keep_images) 3
-set MV(honor_exits) 1
-set MV(urltimeout) 30
-set MV(anim.x_off) 0
-set MV(movespeed) 10
-set MV(honor_call) 1
-set MV(chatsave) 200
-set MV(prevsave) 100
-set MV(timestamp) 0
-set MV(staytime) 10
-set MV(movespeed) 1
-set MV(tooltips) 1
-set MV(rot13) 0
-set MV(bell) 1
-set MV(x) 320
-set MV(y) 200
+set MV(use_windowmanager_colors) 0;	# Do we use our own colors or do we let the window manager set the colors.
+set MV(font.balloon.size) 12;		# The font size to use in balloons.
+set MV(download_avatars) 1;		# Do we download avatars?
+set MV(name_y_offset) 40;		# Our name tag's Y offset (from the image's center)
+set MV(anim.baloon_x) 24;		# Our default X offset for our chat balloon.
+set MV(sort_bookmarks) 0;		# Do we sort our bookmarks by the time visited?
+set MV(name_x_offset) 0;		# Our name tag's X offset (from the image's center)
+set MV(anim.baloon_y) 6;		# Our default Y offset for our chat balloon.
+set MV(sendbuffer) 4096;		# The buffersize to use when sending and receiving files.
+set MV(honor_effects) 1;		# Do we honor avatar effects?
+set MV(font.balloon) -1;		# The font you will use in balloons.
+set MV(history_max) 10;			# Maximum entries in the history list.
+set MV(dcctimeout) 180;			# How many seconds do we allow a DCC to not get data for before we kill it.
+set MV(anim.y_off) 20;			# Our default Y offset for our nametag
+set MV(keep_images) 3;			# How many days do we keep other user's images for?
+set MV(honor_exits) 1;			# Do we honor the server's EXIT protocol commands?
+set MV(urltimeout) 30;			# Seconds URLs will stay on the screen for.
+set MV(anim.x_off) 0;			# Our default X offset for our nametag
+set MV(honor_call) 1;			# Do we honnor calls?
+set MV(chatsave) 200;			# How many lines of chat history do we save?
+set MV(prevsave) 100;			# How many items will be stored in the history for things we sent to the server.
+set MV(TCCommand) 0;			# Special Chat Type Selected.
+set MV(timestamp) 0;			# Do we display time stamps?
+set MV(staytime) 10;			# How many seconds balloons stay on the screen for.
+set MV(movespeed) 1;			# how fast do we move?
+set MV(tooltips) 1;			# Do we setup tooltips for the buttons?
+set MV(rot13) 0;			# Are we using ROt13 encoding?
+set MV(bell) 1;				# Do we beep on sound events?
+set MV(img) -1;				# Our Avatar image's ID number on the canvas.
+set MV(x) 320;				# Our X location on the canvas.
+set MV(y) 200;				# Our Y location on the canvas.
 #
 # The following variables initalize various runtime states and should not
 # be changed at all!! DO NOT CHANGE VARIABLES WITHIN THIS BLOCK.
@@ -118,63 +121,72 @@ set MV(y) 200
 #
 # Strings
 #
-set MV(version) "0.8-1"
-set MV(MyIpAddress) "0.0.0.0"
+set MV(version) "0.8-3";	# OpenVerse Version Number
+set MV(MyIpAddress) "0.0.0.0";	# This will be set to your correct IP Address once you download something.
 #
 # List Variables
 #
-set MV(server_objects) {}
-set MV(server_queries) {}
-set MV(ignore.avatar) {}
-set MV(ignore.effect) {}
-set MV(ignore.speech) {}
-set MV(server_links) {}
-set MV(server_tells) {}
-set MV(server_exits) {}
-set MV(ignore.move) {}
-set MV(ignore.sub) {}
-set MV(ignore.url) {}
-set MV(ignore.all) {}
-set MV(downloads) {}
-set MV(roomloads) {}
-set MV(people) {}
-set MV(chats) {}
-set MV(moves) {}
-set MV(prev) {}
-set MV(dcc_list) {}
+set MV(server_objects) {};	# A list of generic objects (multiple types)
+set MV(server_queries) {};	# Used when querying servers. Contains a list of numbers for servers queried.
+set MV(ignore.avatar) {};	# A list of users who's avatars we are ignoring.
+set MV(ignore.effect) {};	# A list of users who we are ignoring effects for.
+set MV(ignore.speech) {};	# A list of users who we are not listening to.
+set MV(server_links) {};	# A list of LINK objects.
+set MV(server_tells) {};	# A list of server TELL objects.
+set MV(server_exits) {};	# A list of EXIT_OBJ objects.
+set MV(ignore.move) {};		# A list of users who's MOVE protocol commands are ignored.
+set MV(ignore.sub) {};		# A list of uasers who's SUB protocol commands we are ignoring.
+set MV(ignore.url) {};		# A list od users who's URLs we are ignoring.
+set MV(ignore.all) {};		# A list of the people who we have chosen to ignore ALL types of messages from.
+set MV(downloads) {};		# A list of downloads in progress.
+set MV(roomloads) {};		# A list of room images which are downloading.
+set MV(plugins) {};		# A list of plugins installed.
+set MV(people) {};		# A list of user names for everyone who is connected.
+set MV(chats) {};		# This is the chat history, defaults to 200 lines.
+set MV(moves) {};		# A list of users moves which are qued up.
+set MV(prev) {};		# A list of all the stuff we've sent to the server.
+set MV(dcc_list) {};		# A list of active downloads.
 #
 # Numeric Variables
 #
-set MV(default_avatar_number) 0
-set MV(current_exits_id) 0
-set MV(downloadingroom) 0
-set MV(textwindow.lock) 0
-set MV(current_link_id) 0
-set MV(current_tell_id) 0
-set MV(anim.current) -1
-set MV(popup_window) 0
-set MV(anim.running) 0
-set MV(pwq.queries) 0
-set MV(textwindow) 0
-set MV(query_time) 0
-set MV(killsteam) 0
-set MV(dlwindows) 0
-set MV(killnotes) 0
-set MV(killsmile) 0
-set MV(killfrown) 0
-set MV(anim.stop) 1
-set MV(killhigh) 0
-set MV(killlove) 0
-set MV(killidea) 0
-set MV(killwink) 0
-set MV(curprev) -1
-set MV(dcc_num) 0
-set MV(moving) 0
-set MV(dlnum) 0
-set MV(names) 1
-set MV(sock) -1
-set MV(debug) 0
-set MV(log) 0
+set MV(default_avatar_number) 0;	# Which avatar in the list is our default one?
+set MV(debug.scroll.other) 1;	# Do we scroll the debug 'other' section?
+set MV(debug.scroll.prot) 1;	# Do we scroll the debug 'protocol' section?
+set MV(current_exits_id) 0;	# What is the id for the next exit object we create.
+set MV(downloadingroom) 0;	# Are we downloading a room image?
+set MV(textwindow.lock) 0;	# Our scroll lock option within the Text Chat window.
+set MV(current_link_id) 0;	# What is the id for the next link object we create. 
+set MV(current_tell_id) 0;	# What is the id for the next tell object we create.
+set MV(anim.current) -1;	# The avtar number which we are wearing.
+set MV(popup_window) 0;		# Used for error dialogs. Incrementing number.
+set MV(anim.running) 0;		# If our avatar is animated, is it currently running?
+set MV(pwq.queries) 0;		# Used by the Passageways module and is the number of queries which we activated to the servers.
+set MV(textwindow) 0;		# Is the Text Chat window open?
+set MV(killsteam) 0;		# ID used for holding the name of the person who belongs to a steam image.
+set MV(killnotes) 0;		# ID used for holding the name of the person who belongs to a music notes image.
+set MV(killsmile) 0;		# ID used for holding the name of the person who belongs to a smile image.
+set MV(killfrown) 0;		# ID used for holding the name of the person who belongs to a frown image.
+set MV(anim.stop) 1;		# Are we going to stop the animation next time it's scheduled to run?
+set MV(killhigh) 0;		# ID used for holding the name of the person who belongs to a chat balloon.
+set MV(killlove) 0;		# ID used for holding the name of the person who belongs to a love image.
+set MV(killidea) 0;		# ID used for holding the name of the person who belongs to a idea image.
+set MV(killwink) 0;		# ID used for holding the name of the person who belongs to a wink image.
+set MV(curprev) -1;		# This tells us if we are scrolling forwards or backwards in our sent chat history.
+set MV(dcc_num) 0;		# The number of the next DCC transfer we will do.
+set MV(moving) 0;		# Are we moving?
+set MV(names) 1;		# Do we show user nametags?
+set MV(sock) -1;		# This is our socket to the server.
+set MV(debug) 0;		# Are we in debug mode?
+set MV(log) 0;			# Are we logging things?
+#
+# Window name variables.
+#
+set MV(avatar_menu) "ERROR";		# This is the container for the avatars menu button. Your pannel should set this.
+set MV(chat_entry) "ERROR";		# This is the container for the chat entry widget. Your pannel should set this.
+set MV(connect_button) "ERROR";		# This is the container for the connect button widget. Your pannel should set this.
+set MV(names_menu) "ERROR";		# This is the container for the Names menu button widget. Your pannel should set this.
+set MV(passageways_menu) "ERROR";	# This is the container for the Passageways menu button widget. Your pannel should set this.
+set MV(plugin_frame) "ERROR";		# This is the container for plugins to use to put any buttons they may need. Your pannel should set this.
 #
 # End State Initialization Section.
 #
@@ -182,7 +194,7 @@ set MV(log) 0
 #
 # Panel Related
 #
-set MV(panel) "Fun"
+set MV(panel) "Fun";			# The name of the Panel which we are using.
 
 # ------------------------------------------------------------------------
 #                          Plugin Specific Variables.
@@ -194,6 +206,12 @@ set MV(panel) "Fun"
 set MV(plugin.traps.ProcChat.pre) {}
 set MV(plugin.traps.ProcChat.post) {}
 
+# ------------------------------------
+# endDCC {transfer_type download_index success_flag message}
+# ------------------------------------
+set MV(plugin.traps.endDCC.pre) {}
+set MV(plugin.traps.endDCC.post) {}
+
 # ------------------------------------
 # ChangeAvatar {filename}
 # ------------------------------------
@@ -214,11 +232,22 @@ set MV(plugin.traps.ChangeUserAvatar) {}
 # ------------------------------------
 set MV(plugin.traps.Disconnect) {}
 
+# ------------------------------------
+# Debug_memory {file_descriptor}
+# ------------------------------------
+set MV(plugin.traps.Debug_memory) {}
+
 # ------------------------------------
 # DoNames {widget who idx} (will call for each name in list.)
 # ------------------------------------
 set MV(plugin.traps.DoNames) {}
 
+# ------------------------------------
+# Exit {host port}
+# ------------------------------------
+set MV(plugin.traps.Exit.pre) {}
+set MV(plugin.traps.Exit.post) {}
+
 # ------------------------------------
 # ShowName {who}
 # ------------------------------------
@@ -282,8 +311,6 @@ for {set c 0} {$c < 16} {incr c} {
 		set MV(URLS.$c.$d) 0
 	}
 }
-set MV(url.icon.unknown) "unknown.gif"
-set MV(url.txtlen.unknown) 24
 
 # ------------------------------------------------------------------------
 #                       Our default color scheme.
@@ -305,6 +332,28 @@ set MV(plugin.traps.ProcessInput.post) {}
 # ------------------------------------
 set MV(plugin.traps.SendText) {}
 
+# ------------------------------------------------------------------------
+#                       Balloon Fonts available.
+# ------------------------------------------------------------------------
+#
+# Build a list from available fonts.
+#
+set MV(font.balloon.-1.name) ""
+set MV(font.balloon.-1.normal) ""
+set MV(font.balloon.-1.bold) "-weight bold"
+set MV(font.balloon.-1.italic) "-slant italic"
+set MV(font.balloon.-1.bold_italic) "-slant italic -weight bold"
+set MV(font_max) 0
+foreach family [lsort [font families]] {
+	set MV(font.balloon.$MV(font_max).name) "$family"
+	set MV(font.balloon.$MV(font_max).normal) "-family \"$family\""
+	set MV(font.balloon.$MV(font_max).bold) "-family \"$family\" -weight bold"
+	set MV(font.balloon.$MV(font_max).italic) "-family \"$family\" -slant italic"
+	set MV(font.balloon.$MV(font_max).bold_italic) "-family \"$family\" -slant italic -weight bold"
+	incr MV(font_max)
+}
+incr MV(font_max) -1
+
 # ------------------------------------------------------------------------
 #                       Our default color scheme.
 # ------------------------------------------------------------------------
@@ -483,9 +532,9 @@ set MV(colors.dl.buttons.afg) "red"
 #
 for {set c 0} {$c < 26} {incr c} {
 	for {set s 1} {$s < 4} {incr s} {
-		set MV(dls.$c.$s) 0
-		set MV(dlcps.$c.$s) 0
-		set MV(dlactive.$c.$s) 0
+		set MV(dls.$c.$s) 0;		# Status Window ID.
+		set MV(dlcps.$c.$s) 0;		# CPS for this transfer.
+		set MV(dlactive.$c.$s) 0;	# Is this download active?
 	}
 }
 global tl
