@@ -46,7 +46,7 @@ proc SendToServer {what} {
 	set new_what [split $what "\n"]
 	set what [string trim [join $new_what " "]]
 
-	DebugIt "-> $what"
+	DebugIt "-> $what" prot
 	catch {
 		puts $MV(sock) "$what"
 		flush $MV(sock)
@@ -63,12 +63,18 @@ proc ReadFrom {} {
 	catch {gets $MV(sock) input} error
 	if {![string compare $input "NAMEINUSE"]} {
 		ProcChat $MV(nick) "The nick you have chosen is in use. Please choose another. You can use the setup button to do this or type '/nick YourNick' in the chat windows." 0 1 $MV(colors.system.baloon)
-		DebugIt "<- Disconnected!"
+		DebugIt "<- Disconnected!" prot
+		Disconnect
+		return
+	}
+	if {![string compare $input "ROOMFULL"]} {
+		ProcChat $MV(nick) "This room is FULL. Please try another room." 0 1 $MV(colors.system.baloon)
+		DebugIt "<- Disconnected!" prot
 		Disconnect
 		return
 	}
 	if {[eof $MV(sock)] == 1} {
-		DebugIt "<- Disconnected!"
+		DebugIt "<- Disconnected!" prot
 		Disconnect
 		return;
 	}
@@ -277,7 +283,7 @@ proc ProcessCommand {what} {
 			RotTextChat
 		}
 		"/rot13" {
-			DebugIt "Rot13 --- $MV(rot13)"
+			DebugIt "Rot13 --- $MV(rot13)" other
 			if $MV(rot13) {
 				set MV(rot13) 0
 				InsertIntoChat 0 "\0030,4***SYSTEM***\00399,99 Rot13 Encoding Disabled.\n"
@@ -285,7 +291,7 @@ proc ProcessCommand {what} {
 				set MV(rot13) 1
 				InsertIntoChat 0 "\0030,4***SYSTEM***\00399,99 Rot13 Encoding Enabled.\n"
 			}
-			DebugIt "Rot13 --- $MV(rot13)"
+			DebugIt "Rot13 --- $MV(rot13)" other
 		}
 		"/push" {
 			SendToServer "PUSH 100"
@@ -338,20 +344,7 @@ proc ProcessCommand {what} {
 proc Connect {} {
 	global MV
 
-	if $MV(downloadingroom) {
-		tk_dialog .connecT_error "Cannot Change Rooms" \
-		"You cannot change rooms until the room image which you are currently in finishes downloading. You'll notice when this happens :)" \
-		"" 0 "Ok, Thanks for lettine me know!"
-		return
-	}
-	set MV(sock) [socket $MV(roomhost) $MV(roomport)]
-	fconfigure $MV(sock) -blocking 0 -buffering line
-	set args [fconfigure $MV(sock) -sockname]
-	InsertIntoChat 0 "\0030,4***SYSTEM***\00399,99 Connecting to $MV(roomhost) $MV(roomport)\n"
-	set MV(MyIpAddress) [lindex $args 0]
-	fileevent $MV(sock) readable ReadFrom
-	SendToServer "AUTH $MV(nick) $MV(x) $MV(y) $MV(avatar) $MV(anim.x_off) $MV(anim.y_off) [file size "$MV(images)/$MV(avatar)"] $MV(anim.baloon_x) $MV(anim.baloon_y)"
-	$MV(connect_button) configure -text "Disconnect" -command Disconnect
+	ConnectToRoom $MV(roomhost) $MV(roomport)
 }
 
 #
@@ -361,11 +354,15 @@ proc ConnectToRoom {host port} {
 	global MV
 
 	if $MV(downloadingroom) {
-		tk_dialog .connecT_error "Cannot Change Rooms" \
-		"You cannot change rooms until the room image which you are currently in finishes downloading. You'll notice when this happens :)" \
-		"" 0 "Ok, Thanks for lettine me know!"
+		ThrowError "You cannot change rooms until the room image which you are currently in finishes downloading. You'll notice when this happens :)"
 		return
 	}
+	#
+	# Proces Plugins!
+	#
+	foreach plugin $MV(plugin.traps.Exit.pre) {
+		if ![$MV(plugin.traps.Exit.pre.$plugin)] {return}
+	}	
 	ProcChat $MV(nick) "Changing Rooms to $host $port" 0 1 $MV(colors.system.baloon)
 	Disconnect
 	set MV(roomhost) $host
@@ -379,6 +376,12 @@ proc ConnectToRoom {host port} {
 	fileevent $MV(sock) readable ReadFrom
 	SendToServer "AUTH $MV(nick) $MV(x) $MV(y) $MV(avatar) $MV(anim.x_off) $MV(anim.y_off) [file size "$MV(images)/$MV(avatar)"] $MV(anim.baloon_x) $MV(anim.baloon_y)"
 	$MV(connect_button) configure -text "Disconnect" -command Disconnect
+	#
+	# Proces Plugins!
+	#
+	foreach plugin $MV(plugin.traps.Exit.post) {
+		if ![$MV(plugin.traps.Exit.post.$plugin)] {return}
+	}	
 }
 
 #
@@ -403,26 +406,12 @@ proc Disconnect {} {
 	foreach exit $MV(server_exits) {
 		KillServerExit $MV(server_exits.$exit.id)
 	}
+	foreach tell $MV(server_tells) {
+		KillServerTell $MV(server_tells.$tell.id)
+	}
 	foreach who $MV(people) {
-		unset MV($who.name)
-		unset MV($who.avatar)
-		unset MV($who.x)
-		unset MV($who.y)
-		unset MV($who.lastsaid)
-		unset MV($who.name_x_offset)
-		unset MV($who.name_y_offset)
-		unset MV($who.baloon_x)
-		unset MV($who.baloon_y)
-		unset MV($who.haschat)
-		unset MV($who.nomoremove)
-		set MV($who.chatque) {}
-		set idx [lsearch -exact $MV(people) $who]
-		set MV(people) [lreplace $MV(people) $idx $idx]
-		.top.c delete $MV($who.icon)
-		if $MV(names) {KillName $who}
+		PersonLeft $who
 	}
-	DoNames
-
 	#
 	# Proces Plugins!
 	#
@@ -437,7 +426,7 @@ proc Disconnect {} {
 proc ProcessInput {what} {
 	global MV
 
-	DebugIt "<- $what"
+	DebugIt "<- $what" prot
 	set cmd [string range $what 0 [expr [string first " " $what] -1]]
 	set rest [string range $what [expr [string first " " $what] +1] end]
 	set parms [split $rest " "]
@@ -490,7 +479,7 @@ proc ProcessInput {what} {
 					ProcChat [lindex $parms 1] $rest 0 9 $MV(colors.chat.baloon)
 				}
 				"WINK" {
-					ProcChat [lindex $parms 1] $rest 0 10 $MV(colors.chat.baloon)
+					ProcChat [lindex $parms 1] $rest 0 A $MV(colors.chat.baloon)
 				}
 				default {
 					ProcChat [lindex $parms 1] $rest 0 0 $MV(colors.chat.baloon)
@@ -538,6 +527,14 @@ proc ProcessInput {what} {
 			SendToServer "PONG"
 		}
 		"PUSH" {
+			#
+			# Validate here because of the plugin support here.
+			#
+			if {[TestNum [lindex $parms 0]] || [TestNum [lindex $parms 1]]} {
+				DebugIt "PUSH Parameters invalid." other
+				return
+			}
+
 		        # Plugin support added, KaosBeetl 02/09/2000
 			#
 			# Proces Plugins!
@@ -680,6 +677,13 @@ proc ProcessInput {what} {
  				[lindex $parms 4] [lindex $parms 5] \
 				[lindex $parms 6] $text
 		}
+		"MOVE_OBJ" {
+			# Move an object.
+			# MOVE_OBJ NAME x_offset y_offset type speed
+			MoveObject [lindex $parms 0] [lindex $parms 1] \
+				[lindex $parms 2] [lindex $parms 3] \
+				[lindex $parms 4]
+		}
 	}
 		# --------------------------------------------------
 		# Server "Objects"
@@ -693,6 +697,7 @@ proc ProcessInput {what} {
 		# TELL		NAME x1 y1 x2 y2 duration
 		# ENTRY		NAME x1 y1 length bg_color text_color duration |default text value
 		# SUBMIT	NAME x1 y1 duration |text
+		# MOVE_OBJ 	NAME x_offset y_offset type speed
 		# --------------------------------------------------
 	#
 	# Proces Plugins!
@@ -751,7 +756,13 @@ proc ShowDownloadStatus {row col} {
 	set leftx [expr $xx -66]
 
 	.top.c delete $MV(dlsts.$row.$col)
-	set MV(dlsts.$row.$col) [.top.c create rectangle $leftx [expr $y + 5] [expr $leftx + $rightx] [expr $y + 14] -fill red]
+	set idx $MV(dlidx.$row.$col)
+	set type $MV(DCC.$idx.io)
+	if {![string compare $type "Get"]} {
+		set MV(dlsts.$row.$col) [.top.c create rectangle $leftx [expr $y + 5] [expr $leftx + $rightx] [expr $y + 14] -fill green]
+	} else {
+		set MV(dlsts.$row.$col) [.top.c create rectangle $leftx [expr $y + 5] [expr $leftx + $rightx] [expr $y + 14] -fill red]
+	}
 	if {[winfo exists .top.c.dlcanc]} {
 		if $MV(dlactive.$row.$col) {
 			set idx $MV(dlidx.$row.$col)
@@ -784,7 +795,7 @@ proc DCCGet {host port file size type sender} {
 	global MV
 
 	if ![SanityCheck "$file"] {
-		DebugIt "(!) $file fails SanityCheck"
+		DebugIt "(!) $file fails SanityCheck" other
 		return
 	}
 
@@ -813,6 +824,7 @@ proc DCCGet {host port file size type sender} {
 
 	set idx [incr MV(dcc_num)]	
 	set MV(DCC.$idx.sender) $sender
+	set MV(DCC.$idx.io) "Get"
 	set MV(DCC.$idx.file) "$dldir/$file"
 	set MV(DCC.$idx.size) $size
 	set MV(DCC.$idx.type) $type
@@ -869,7 +881,7 @@ proc dccgevent {index st out} {
 	set fail_type 0
 
     if {[eof $in]} {
-	DebugIt "(!) Dcc Ended, EOF detected."
+	DebugIt "(!) Dcc Ended, EOF detected." prot
         if {$tl($in) < $leng} {
 	    set msg "Transfer Interrupted"
 		set fail_type 0
@@ -889,7 +901,7 @@ proc dccgevent {index st out} {
     } {
         if {![catch {set buffer [read $in]} msg]} {
             incr tl($in) [set l [string length $buffer]]
-		DebugIt "downloaded $l bytes ($tl($in) total) wrote to $out $MV(DCC.$index.file)"
+		DebugIt "downloaded $l bytes ($tl($in) total) wrote to $out $MV(DCC.$index.file)" prot
             if {[set dt [expr {[clock seconds] - $st}]] == 0 || $tl($in) == 0} {
                 set elt 0
             } {
@@ -923,7 +935,7 @@ proc dccgevent {index st out} {
 	    } else {
 		set msg "Unable to report received size"
 		set fail_type 0
-		DebugIt "(!) Dcc Ended, Error Condition."
+		DebugIt "(!) Dcc Ended, Error Condition." prot
 	    }
 	} else {
 		# 0000 bytes triggered a read event?
@@ -934,7 +946,7 @@ proc dccgevent {index st out} {
 	   # "(!) Dcc Ended, Error Condition."
 	}
     }
-    DebugIt "closing $out $MV(DCC.$index.file)"
+    DebugIt "closing $out $MV(DCC.$index.file)" prot
     catch {close $out} 
     endDCC Get $index $fail_type $msg
 }
@@ -954,7 +966,7 @@ proc DCCSendAv {host port what} {
 
 	if {[file exists $file]} {
 		if {![file readable $file]} {
-            		DebugIt "($who) (DCCSend) Cannot read file $file."
+            		DebugIt "($who) (DCCSend) Cannot read file $file." prot
 			return
 		}
 
@@ -964,6 +976,12 @@ proc DCCSendAv {host port what} {
 		fconfigure $sock -blocking 1
 
 		set MV(DCC.$idx.sender) "*"
+		set MV(DCC.$idx.io) "Send"
+		#
+		set MV(DCC.$idx.notify) "NONE"
+		set MV(DCC.$idx.cps) 0
+		set MV(DCC.$idx.status) 0
+		#
 		set MV(DCC.$idx.file) "$file"
 		set MV(DCC.$idx.size) $size
 		set MV(DCC.$idx.posn) 0
@@ -982,7 +1000,7 @@ proc DCCSendAv {host port what} {
 		lappend MV(dcc_list) $idx
 		fileevent $sock writable "acceptAVSend $idx"
 	} else {
-		DebugIt "($who) (DCCSendAv) File $file does not exist."
+		DebugIt "($who) (DCCSendAv) File $file does not exist." prot
 	}
 	update idletasks
 }
@@ -1031,7 +1049,7 @@ proc acceptAVSend {index} {
 		return 0
 	}
 	flush $chan
-	DebugIt "($MV(DCC.$index.sender)) -- Accepted DCCSend"
+	DebugIt "($MV(DCC.$index.sender)) -- Accepted DCCSend" prot
 	fileevent $chan readable "dccSendEvent $index $st $infile"
 }
 
@@ -1041,22 +1059,23 @@ proc acceptAVSend {index} {
 # by Lindsay Marshall <lindsay.marshall@newcastle.ac.uk>
 #
 # This is our central location for SENDING ALL files.
-
+# 
+# This function will return the index of the file which is being sent if
+# all went well, otherwise it will return -1. This index can be used to
+# set the MV(DCC.$idx.notify) variable to a function name which will be
+# called to let a plugin or remote application know that the download has
+# finished and with what status.
+#
 proc DCCSend {who what} {
 	global MV
 
-	#if ![VerifyAvailable $who $what] {
-		#puts "($who) -- Already Getting $what"
-		#return
-	#}
-
 	set file "$what"
 	set what "[file tail $file]"
 
 	if {[file exists $file]} {
 		if {![file readable $file]} {
-            		DebugIt "($who) (DCCSend) Cannot read file $file."
-			return
+            		DebugIt "($who) (DCCSend) Cannot read file $file." prot
+			return -1
 		}
 
 		set size [file size $file]
@@ -1064,11 +1083,17 @@ proc DCCSend {who what} {
 		set sock [socket -server "acceptSend $idx" 0]
 
 		if {[catch {fconfigure $sock -sockname} port]} {
-			DebugIt "($who) (DCCSend) Cannot get port for server - $port"
+			DebugIt "($who) (DCCSend) Cannot get port for server - $port" prot
 		}
 
 		set MV(DCC.$idx.sender) $who
 		set MV(DCC.$idx.type) "OUTBOUND SEND"
+		set MV(DCC.$idx.io) "Send"
+		#
+		set MV(DCC.$idx.notify) "NONE"
+		set MV(DCC.$idx.cps) 0
+		set MV(DCC.$idx.status) 0
+		#
 		set MV(DCC.$idx.file) "$file"
 		set MV(DCC.$idx.size) $size
 		set MV(DCC.$idx.posn) 0
@@ -1086,8 +1111,10 @@ proc DCCSend {who what} {
 		lappend MV(dcc_list) $idx
 	
 		SendToServer "SUB $who DCCGET $MV(MyIpAddress) [lindex $port 2] $what $size"
+		return $idx
 	} else {
-		DebugIt "($who) (DCCSend) File $file does not exist."
+		DebugIt "($who) (DCCSend) File $file does not exist." prot
+		return -1
 	}
 }
 
@@ -1146,18 +1173,21 @@ proc dccSendEvent {index st fd} {
 	uplevel #0 set MV(DCC.$index.time) [clock seconds]
 
 	if {[eof $sk]} {
+		set MV(DCC.$index.status) -1
 		endDCC Send $index 0 "Transfer interrupted"
 		close $fd
 		return
 	}
 
 	if {[catch {set l [read $sk 4]} msg]} {
+		set MV(DCC.$index.status) -1
 		endDCC Send $index 0 "Read error : $msg"
 		close $fd
 		return
 	}
 
 	if {[string length $l] == 0} {
+		set MV(DCC.$index.status) -1
 		endDCC Send $index 0 "Sync read error"
 		close $fd
 		return
@@ -1173,11 +1203,13 @@ proc dccSendEvent {index st fd} {
 			    set st 1
 		}
 		close $fd
+		set MV(DCC.$index.status) 1
 		endDCC Send $index 1 "Transfer completed"
 		return
 	}
 
 	if {[catch {set buffer [read $fd $MV(sendbuffer)]} msg]} {
+		set MV(DCC.$index.status) -1
 		endDCC Send $index 0 "Error reading $MV(DCC.$index.file) : $msg"
 		close $fd
 		return
@@ -1188,11 +1220,13 @@ proc dccSendEvent {index st fd} {
 			set st 1
 		}
 		close $fd
+		set MV(DCC.$index.status) 1
 		endDCC Send $index 1 "Transfer completed."
 		return
 	}
 	incr tl($sk) $lng
 	if {[catch {puts -nonewline $sk $buffer} msg]} {
+		set MV(DCC.$index.status) -1
 		endDCC Send $index 0 "Write error : $msg"
 		close $fd
 		return
@@ -1216,11 +1250,11 @@ proc dccSendEvent {index st fd} {
 }
 
 proc endDCC {type index fail_type debug} {
-	global MV
+	global MV tl
 
 	set idx [lsearch -exact $MV(downloads) [file tail $MV(DCC.$index.file)]]
 	if {$index >= 0} {set MV(downloads) [lreplace $MV(downloads) $idx $idx]}
-	DebugIt "(!) (DCC$type) - $debug $MV(DCC.$index.file)"
+	DebugIt "(!) (DCC$type) - $debug $MV(DCC.$index.file)" prot
 	catch {close $MV(DCC.$index.sock)}
 	if {$MV(DCC.$index.server) > 0} {
 		catch {close $MV(DCC.$index.server)}
@@ -1229,18 +1263,27 @@ proc endDCC {type index fail_type debug} {
 	# Kaos's FIX
 	# ISSUE 01
 	#
+	#
+	# Proces Plugins!
+	#
+	#foreach plugin $MV(plugin.traps.endDCC.pre) {
+		#if ![$MV(plugin.traps.endDCC.pre.$plugin) "$type" $index "$fail_type" "$debug"] {return}
+	#}
  	if {![string compare $type "Cancel"] && ![string compare $MV(DCC.$index.type) "ROOM"]} {
 		set MV(downloadingroom) 0
 	}
+	if {![string compare $type "Send"] && [string compare $MV(DCC.$index.notify) "NONE"]} {
+		$MV(DCC.$index.notify) $type $index $fail_type $debug
+	}
 	if {![string compare $type "Get"]} {
 		switch -exact -- $MV(DCC.$index.type) {
 			"AVATAR" {
 				if $fail_type {
 					foreach who $MV(people) {
 						if {![string compare $MV($who.avatar) [file tail $MV(DCC.$index.file)]] && $MV($who.downloading) == 1} {
-							catch {image create photo $who -file "$MV(rem_images)/$MV($who.avatar)"} err
-							if {[string compare $err $who]} {
-								catch {image create photo $who -file "$MV(images)/default.gif"} err
+							catch {image create photo OpenVerse_User_Image_$who -file "$MV(rem_images)/$MV($who.avatar)"} err
+							if {[string compare $err OpenVerse_User_Image_$who]} {
+								catch {image create photo OpenVerse_User_Image_$who -file "$MV(images)/default.gif"} err
 							}
 							set MV($who.downloading) 0
 						}
@@ -1255,7 +1298,7 @@ proc endDCC {type index fail_type debug} {
 						if {![string compare $MV(server_objects.$object.image) [file tail $MV(DCC.$index.file)]]} {
 							catch {image create photo server_object_$object -file "$MV(objects)/$MV(server_objects.$object.image)"} err
 							if {[string compare $err "server_object_$object"]} {
-								DebugIt "$err"
+								DebugIt "$err" prot
 								catch {image create photo server_object_$object -file "$MV(icons)/unknown.gif"} err
 							}
 						}
@@ -1267,7 +1310,7 @@ proc endDCC {type index fail_type debug} {
 			"ROOM" {
 				if $fail_type {
 					set MV(downloadingroom) 0
-					catch {image create photo room -file "$MV(roomdir)/[file tail $MV(DCC.$index.file)]"}
+					catch {image create photo OpenVerse_Image_room -file "$MV(roomdir)/[file tail $MV(DCC.$index.file)]"}
 				} else {
 					SendToServer "DCCSENDROOM [file tail $MV(DCC.$index.file)]"
 				}
@@ -1278,6 +1321,38 @@ proc endDCC {type index fail_type debug} {
 	if {[winfo exists .top.c.dlcanc]} {KillDlInfoWin $MV(DCC.$index.row) $MV(DCC.$index.col)}
 	set idx [lsearch -exact $MV(dcc_list) $index]
 	set MV(dcc_list) [lreplace $MV(dcc_list) $idx $idx]
+	#
+	# Proces Plugins!
+	#
+	foreach plugin $MV(plugin.traps.endDCC.post) {
+		$MV(plugin.traps.endDCC.post.$plugin) "$type" $index "$fail_type" "$debug"
+	}
+	# Cleanup memory used by this transfer.
+	catch {unset MV(dlbounds.$MV(DCC.$index.row).$MV(DCC.$index.col))}
+	catch {unset MV(dlbox.$MV(DCC.$index.row).$MV(DCC.$index.col))}
+	catch {unset MV(dlidx.$MV(DCC.$index.row).$MV(DCC.$index.col))}
+	catch {unset MV(dlperc.$MV(DCC.$index.row).$MV(DCC.$index.col))}
+	catch {unset MV(dlsts.$MV(DCC.$index.row).$MV(DCC.$index.col))}
+	catch {unset MV(dlstsbx.$MV(DCC.$index.row).$MV(DCC.$index.col))}
+	catch {unset MV(dltxt.$MV(DCC.$index.row).$MV(DCC.$index.col))}
+	catch {unset MV(DCC.$MV(DCC.$index.row).$MV(DCC.$index.col).window)}
+	catch {set MV(dlcps.$MV(DCC.$index.row).$MV(DCC.$index.col)) 0}
+	catch {unset tl($MV(DCC.$index.sock))}
+	catch {unset MV(DCC.$index.col)}
+	catch {unset MV(DCC.$index.file)}
+	catch {unset MV(DCC.$index.io)}
+	catch {unset MV(DCC.$index.posn)}
+	catch {unset MV(DCC.$index.row)}
+	catch {unset MV(DCC.$index.sender)}
+	catch {unset MV(DCC.$index.server)}
+	catch {unset MV(DCC.$index.size)}
+	catch {unset MV(DCC.$index.cps)}
+	catch {unset MV(DCC.$index.notify)}
+	catch {unset MV(DCC.$index.port)}
+	catch {unset MV(DCC.$index.status)}
+	catch {unset MV(DCC.$index.sock)}
+	catch {unset MV(DCC.$index.time)}
+	catch {unset MV(DCC.$index.type)}
 }
 
 proc CheckTimeouts {} {
@@ -1300,7 +1375,10 @@ proc KillDlInfoWin {row col} {
 
 	set MV(dlactive.$row.$col) 0
 	destroy .top.c.dlcanc
-	.top.c delete $MV(DCC.$row.$col.window)
+	# Bug reported by topie 
+	if {[info exists MV(DCC.$row.$col.window)]} {
+		.top.c delete $MV(DCC.$row.$col.window)
+	}
 }
 
 proc DownloadClick {row col x y} {
@@ -1430,7 +1508,7 @@ proc ProcPersonal {who data} {
 	# data = "ID Data"
 	set id [lindex [split $data] 0]
 	set value [string range $data [expr [string length $id] + 1] end]
-	DebugIt "(PERSONAL) ID=$id DATA=$value"
+	DebugIt "(PERSONAL) ID=$id DATA=$value" other
 	switch -- $id {
 		1 {
 			set MV($who.personal.name) $value
