@@ -53,6 +53,8 @@ proc InitSetVars {} {
 	set SET(staytime) $MV(staytime)
 	set SET(movespeed) $MV(movespeed)
 	set SET(urltimeout) $MV(urltimeout)
+	set SET(font.balloon) $MV(font.balloon)
+	set SET(font.balloon.size) $MV(font.balloon.size)
 	#-------------------------
 	# Strings
 	#-------------------------
@@ -69,21 +71,20 @@ proc InitSetVars {} {
 	set SET(personal.homepage) "$MV(personal.homepage)"
 	set SET(personal.info) "$MV(personal.info)"
 	set SET(personal.availability) "$MV(personal.availability)"
+	set SET(font.balloon.style) "$MV(font.balloon.style)"
 }
 
 #
 # This sets up the setup window.
 #
 proc Setup {} {
-	global SET
-
-	InitSetVars
-
-	set SET(refresh_menu) 0
-
-	image create photo PERSONAL_IMAGE -file [file nativename "$SET(personal.image)"]
+	global SET MV
 
 	if {[winfo exists .setup]} {destroy .setup}
+	InitSetVars
+	set SET(refresh_menu) 0
+	image create photo PERSONAL_IMAGE -file [file nativename "$SET(personal.image)"]
+
 	toplevel .setup
 	wm title .setup "OpenVerse Setup"
 	wm resizable .setup 0 0
@@ -139,6 +140,63 @@ proc Setup {} {
 	label .setup.optb.http_l -text "Browser Cmd" -width 14
 	entry .setup.optb.http_e
 
+	frame .setup.optb.font -relief sunken -borderwidth 2
+	menubutton .setup.optb.font.balloon_font -text "Balloon Font" \
+		-indicatoron 1 -relief raised -borderwidth 1 \
+		-menu .setup.optb.font.balloon_font.m
+	menu .setup.optb.font.balloon_font.m
+	set depth 0
+	set rows 0
+	for {set c 0} {$c <= $MV(font_max)} {incr c} {
+		if {!$depth || $rows > 10} {
+			incr depth
+			set rows 0
+			set menulabel "FontList $depth"
+			.setup.optb.font.balloon_font.m add cascade -menu \
+				.setup.optb.font.balloon_font.m.m$depth \
+				-label $menulabel
+			menu .setup.optb.font.balloon_font.m.m$depth
+		}
+		.setup.optb.font.balloon_font.m.m$depth add radiobutton \
+			-label "$MV(font.balloon.$c.name)" \
+			-variable SET(font.balloon) \
+			-value $c
+		incr rows
+	}
+	menubutton .setup.optb.font.font_size -text "Size" \
+		-indicatoron 1 -relief raised -borderwidth 1 \
+		-menu .setup.optb.font.font_size.m
+	menu .setup.optb.font.font_size.m
+	.setup.optb.font.font_size.m add radiobutton \
+		-label "8" \
+		-variable SET(font.balloon.size) \
+		-value 8
+	.setup.optb.font.font_size.m add radiobutton \
+		-label "10" \
+		-variable SET(font.balloon.size) \
+		-value 10
+	.setup.optb.font.font_size.m add radiobutton \
+		-label "12" \
+		-variable SET(font.balloon.size) \
+		-value 12
+	.setup.optb.font.font_size.m add radiobutton \
+		-label "14" \
+		-variable SET(font.balloon.size) \
+		-value 14
+	.setup.optb.font.font_size.m add radiobutton \
+		-label "16" \
+		-variable SET(font.balloon.size) \
+		-value 16
+
+	radiobutton .setup.optb.font.normal -text "Normal" \
+		-variable SET(font.balloon.style) -value "normal"
+	radiobutton .setup.optb.font.bold -text "Bold" \
+		-variable SET(font.balloon.style) -value "bold"
+	radiobutton .setup.optb.font.italic -text "Italic" \
+		-variable SET(font.balloon.style) -value "italic"
+	radiobutton .setup.optb.font.bold_italic -text "Bold Italic" \
+		-variable SET(font.balloon.style) -value "bold_italic"
+
 	frame .setup.optba.serv
 	label .setup.optba.serv.l -text "Default Server" -width 14
 	entry .setup.optba.serv.e
@@ -275,6 +333,9 @@ proc Setup {} {
 	pack .setup.optb -fill both -expand y
 	pack .setup.optb.http_l -side left
 	pack .setup.optb.http_e -side left -fill both -expand y
+	pack .setup.optb.font -side left -fill both -expand y
+	pack .setup.optb.font.balloon_font .setup.optb.font.font_size .setup.optb.font.normal \
+		.setup.optb.font.bold .setup.optb.font.italic .setup.optb.font.bold_italic -side left
 	pack .setup.optba -fill both -expand y
 	pack .setup.optba.serv -side left -fill both -expand y
 	pack .setup.optba.port -side left
@@ -333,6 +394,7 @@ proc Setup {} {
 		set SET(panel) [selection get]
 		set SET(refresh_menu) 1
 	}
+	bind .setup <Destroy> {Unset_Setup}
 	FillPanels
 
 }
@@ -402,10 +464,13 @@ proc SaveSetSettings {} {
 	puts $outfile "set MV(staytime) $SET(staytime)"
 	puts $outfile "set MV(movespeed) $SET(movespeed)"
 	puts $outfile "set MV(urltimeout) $SET(urltimeout)"
+	puts $outfile "set MV(font.balloon) $SET(font.balloon)"
+	puts $outfile "set MV(font.balloon.size) $SET(font.balloon.size)"
 	#------------------------
 	# Strings
 	#------------------------
 	puts $outfile "set MV(panel) \"$SET(panel)\""
+	puts $outfile "set MV(font.balloon.style) $SET(font.balloon.style)"
 	puts $outfile "set MV(nick) \"[.setup.na.l.t.nick.entry get]\""
 	puts $outfile "set MV(anim) \"[.setup.na.l.b.avatars.entry get]\""
 	puts $outfile "set MV(browser_cmd) \"[.setup.optb.http_e get]\""
@@ -435,7 +500,7 @@ proc ApplySetSettings {} {
 	set MV(names) $SET(names)
 	set MV(tooltips) $SET(tooltips)
 	set MV(timestamp) $SET(timestamp)
-	set MV(download_avatars) $MV(download_avatars)
+	set MV(download_avatars) $SET(download_avatars)
 	set MV(honor_effects) $SET(honor_effects)
 	#------------------------
 	# Numeric
@@ -443,9 +508,12 @@ proc ApplySetSettings {} {
 	set MV(staytime) $SET(staytime)
 	set MV(movespeed) $SET(movespeed)
 	set MV(urltimeout) $SET(urltimeout)
+	set MV(font.balloon) $SET(font.balloon)
+	set MV(font.balloon.size) $SET(font.balloon.size)
 	#------------------------
 	# Strings
 	#------------------------
+	set MV(font.balloon.style) $SET(font.balloon.style)
 	if $SET(refresh_menu) {
 		set MV(panel) "$SET(panel)"
 		source "$MV(paneldir)/$MV(panel)"
@@ -473,3 +541,8 @@ proc ApplySetSettings {} {
 	set MV(personal.info) [.setup.optc.info.meat.txt.t get 0.0 end]
 	set MV(personal.availability) $SET(personal.availability)
 }
+
+proc Unset_Setup {} {
+	global SET
+	catch {unset SET}
+}
