@@ -32,8 +32,6 @@
 #
 # Everything uses this so we must define it first.
 #
-# Checked for maximum Memmory Efficiency on 12/09/1999 13:33
-#
 proc LogIt {text} {
 	global MVS
 
@@ -59,8 +57,6 @@ proc LogIt {text} {
 # This function will send the provided text to all connected users. The
 # text should be pre-formated and ready to go.
 #
-# Checked for maximum memory efficiency 12/09/1999 13:36
-#
 proc SendToAllUsers {what} {
 	global MVS
 	
@@ -104,8 +100,6 @@ if {!$tcl_interactive} {
 	#
 	# Only defined if running standalone. (command line daemon)
 	#
-	# Checked for maximum memmory efficiency 12/09/1999 13:59
-	#
 	proc SanityCheck {what} {
 		if {[string first "../" $what] != -1} {
 			unset what
@@ -137,34 +131,13 @@ if {!$tcl_interactive} {
 	# number.
 	# only defined if running standalone. (command line daemon)
 	#
-	# Checked for maximum memmory efficiency 12/09/1999 13:59
-	#
-	proc TestNum {what} {
+	proc TestNum {number} {
 		global MVS
-		set bad 0
-		for {set c 0} {$c < [string length $what] && !$bad} {incr c} {
-			switch -exact -- [string range $what $c $c] {
-				1 {}
-				2 {}
-				3 {}
-				4 {}
-				5 {}
-				6 {}
-				7 {}
-				8 {}
-				9 {}
-				0 {}
-				- {}
-				default {set bad 1}
-			}
-		}
-		unset c what
-		if $bad {
-			unset bad
+
+		if {[string length [string trim $number -0123456789]]} {
 			return 1
 		} else {
-			unset bad
-			return 0
+			return 0                  
 		}
 	}
 } else {
@@ -181,38 +154,65 @@ if {!$tcl_interactive} {
 # This function will (re)load the server config file. Any changes will
 # take effect at the time of reload.
 #
-# Checked for maximum memory efficiency 12/09/1999 15:35
-#
 proc ReloadConfig {} {
 	global MVS
 
 	LogIt "------------ Loading Config File -----------"
 
-	set MVS(port) "7000"
-	set MVS(timeout) 120
-	set MVS(roomname) "My Own Room"
-	set MVS(avatars) "$MVS(homedir)/simages"
-	set MVS(sobjects) "$MVS(homedir)/sobjects"
+	# ----------------------------
+	# ORT Section
+	# ----------------------------
+	# Config
+	set MVS(ORT_Admin) "Joe Admin";				# The Admin's name
+	set MVS(ORT_AdminEmail) "openverse@openverse.org";	# The Admin's email address.
+	set MVS(ORT_Country) "United States";			# The country this server is in.
+	set MVS(ORT_Description) "Description Not Set!";	# A brief description of this server.
+	set MVS(ORT_Image) "ov_tram_logo.gif";			# Our banner image for the ORT
+	set MVS(ORT_Rating) "PG";				# Our content Rating.
+	set MVS(ORT_WebSite) "http://openverse.org/";		# This server's website address.
+	set MVS(ORT_Server) {};					# A list of ORT server:port values.
+	set MVS(ORT_Username) {};				# A list of ORT username values for each server.
+	set MVS(ORT_Password) {};				# A list of ORT passwords for each server.
+	set MVS(ORT_Location) {};				# A list of ORT screen locations for each server.
+	# Runtime
+	set MVS(ORT_current_ort) -1;				# Is an ORT stopped at out server? -1 is no.
+
+	# Basic Server Settings.
+	# Config
+	set MVS(port) "7000";					# What port the server is runing on.
+	set MVS(timeout) 120;					# Seconds to wait before calling a transfer failed.
+	set MVS(roomname) "My Own Room";			# The name of this room.
+	set MVS(avatars) "$MVS(homedir)/simages";		# Where the remote user's avatars will be placed.
+	set MVS(sobjects) "$MVS(homedir)/sobjects";		# The directory where avatars are stored.
 	set MVS(tickler) "$MVS(homedir)/TickleMe"
+	set MVS(mem_tickler) "$MVS(homedir)/TickleMem"
 	set MVS(images) "$MVS(homedir)/images"
 	set MVS(icons) "$MVS(homedir)/icons"
 	set MVS(ORT_current_ort) -1
-	set MVS(maxheight) "200"
-	set MVS(push) 1
-	set MVS(maxwidth) "320"
-	set MVS(roomdir) "$MVS(homedir)/rooms"
-	set MVS(roomfile) "room.gif"
+	set MVS(maxheight) "200";				# Maximum allowed image height
+	set MVS(push) 1;					# Does the server support pushing?
+	set MVS(maxwidth) "320";				# Maximum Allowed Image Width.
+	set MVS(roomdir) "$MVS(homedir)/rooms";			# The directory where room images are stored.
+	set MVS(roomfile) "room.gif";				# The name (not the full path) of the room image.
 	set MVS(sendbuffer) 4096
 	set MVS(maxmsglen) 256
-	set MVS(exits) {}
-	set MVS(locations) {}
-	set MVS(captains) {}
+	set MVS(exits) {};					# A list of exit server:port entries.
+	set MVS(locations) {};					# A list of coords for the EXIT type exits.
 	set MVS(maxpushdistance) 100
 	set MVS(maxpushvelocity) 100
-	set MVS(ORT_Server) {}
-	set MVS(ORT_Username) {}
-	set MVS(ORT_Password) {}
-	set MVS(ORT_Location) {}
+	set MVS(max_same_users) 10
+	set MVS(register_ort) 1
+
+	# Info vars - created by other functions.
+	# MVS(waiter);				# dummy variable for tclsh looping.
+	# MVS(serving);				# Is the server running? (used mostly for the GUI mode)
+	# MVS(users);				# Number of connected users.
+	# MVS(socks);				# A list of connected users sockets.
+	# MVS(configfile);			# The name of our configuration file.
+	# MVS(dcc_list);			# A list of active DCCs
+	# MVS(dcc_num);				# An ever increasing number for assigning IDs to downloads.
+	# MVS(entry_registry);			# A list of functions to call for entry object events.
+	# MVS(homedir);				# Our home (root) directory.
 	# Create required directories.
 
 	if ![file exists $MVS(avatars)] {file mkdir "$MVS(avatars)"}
@@ -227,6 +227,11 @@ proc ReloadConfig {} {
 			file delete -force "$MVS(tickler)"
 		}
 	}
+	if [file exists "$MVS(mem_tickler)"] {
+		catch {
+			file delete -force "$MVS(mem_tickler)"
+		}
+	}
 	SendToAllUsers "ROOMNAME $MVS(roomname)"
 	SendToAllUsers "ROOM $MVS(roomfile) [file size "$MVS(roomdir)/$MVS(roomfile)"]"
 }
@@ -249,8 +254,6 @@ LogIt "(!) -- SOCKET_OPEN $MVS(server_socket)"
 # variables for the user. It will also set a trigger on the incoming
 # socket which will read data on the socket.
 #
-# checked for maximum memmory efficiency 12/09/1999 18:07
-#
 proc NewConnect {sck address port} {
 	global MVS
 
@@ -279,6 +282,14 @@ proc NewConnect {sck address port} {
 	if !$MVS(standalone) {
 		.ovserver.buttons.info.v configure -text $MVS(users)
 	}
+	set count 0
+	foreach sock $MVS(socks) {
+		if {$MVS($sock.address) == $address} {incr count}
+	}
+	if {$count > $MVS(max_same_users)} {
+		SendToUser $sck "TOOMANYCONNECTIONS"
+		DisconnectUser $sck 0
+	}
 	unset sck address port
 }
 
@@ -290,8 +301,6 @@ proc NewConnect {sck address port} {
 # this function will check a given nickname to be sure it is allowed. Some
 # nicknames are not allowed if they contain special characters.
 #
-# Checked for maximum memmory efficiency 12/09/1999 18:20
-#
 proc CheckName {name} {
 	if {[string trim $name] == "" || \
 		[string trim $name] == "*" || \
@@ -315,8 +324,6 @@ proc CheckName {name} {
 # request. If it has nothing to do other than to read the text, it will
 # pass this text off to the Serv_ProcessInput function for processing.
 #
-# Checked for maximum memmory efficience on 12/09/1999 18:27
-#
 proc Serv_ReadFrom {who} {
 	global MVS
 	
@@ -424,7 +431,7 @@ proc Serv_ReadFrom {who} {
 				} else {
 					set image "$MVS(ORT_info.$MVS(ORT_current_ort).image)"
 				}
-				SendToUser $who "NEW $MVS(ORT_info.$MVS(ORT_current_ort).name) $MVS(ORT_info.$MVS(ORT_current_ort).x) $MVS(ORT_info.$MVS(ORT_current_ort).y) $image 0 60 [file size $MVS(avatars)/$image] 0 100"
+				SendToUser $who "NEW $MVS(ORT_info.$MVS(ORT_current_ort).name) $MVS(ORT_info.$MVS(ORT_current_ort).x) $MVS(ORT_info.$MVS(ORT_current_ort).y) $image 0 60 [file size $MVS(avatars)/$image] 66 -44"
 				set a $MVS(ORT_info.$MVS(ORT_current_ort).x)
 				set b $MVS(ORT_info.$MVS(ORT_current_ort).y)
 				SendToUser $who "EXIT_OBJ ov_tram_exit [expr $a - 60] [expr $b - 60] [expr $a + 60] [expr $b + 60] 0 $MVS(ORT_info.$MVS(ORT_current_ort).host) $MVS(ORT_info.$MVS(ORT_current_ort).port)"
@@ -472,6 +479,9 @@ proc Serv_ReadFrom {who} {
 	unset input who
 }
 
+# Authenticate transport systems
+#
+# Usage: TransAuth user_socket parms_list
 #
 # This function will authenticate transport systems.
 #
@@ -526,6 +536,13 @@ proc TransAuth {who what} {
 	}
 }
 
+# Display Transport System
+# 
+# Usage: TransAuth index image name time port
+#
+# This function will display the ORT to the users. It will schedule it to
+# be destroyed later.
+#
 proc DisplayORT {idx image name time port} {
 	global MVS
 
@@ -536,10 +553,21 @@ proc DisplayORT {idx image name time port} {
 	set y [lindex $parms 1]
 	set server_parms [split [lindex $MVS(ORT_Server) $idx] ":"]
 
+	set unique 1
+	foreach sck $MVS(socks) {
+		if {![string compare $name $MVS($sck.name)]} {
+			set unique 0
+		}
+	}
+
 	set MVS(ORT_info.$idx.image) $image
+	if {!$unique} {
+		append name "_(REAL)"
+	}
 	set MVS(ORT_info.$idx.name) $name
 	set MVS(ORT_info.$idx.host) [lindex $server_parms 0]
 	set MVS(ORT_info.$idx.port) $port
+	set MVS(ORT_info.$idx.time) $time
 	set MVS(ORT_info.$idx.x) [lindex $parms 0]
 	set MVS(ORT_info.$idx.y) [lindex $parms 1]
 	set MVS(ORT_current_ort) $idx
@@ -547,17 +575,46 @@ proc DisplayORT {idx image name time port} {
 	if {![file exists "$MVS(avatars)/$image"]} {
 		set image "default.gif"
 	}
-	SendToAllUsers "NEW $name [lindex $parms 0] [lindex $parms 1] $image 0 60 [file size $MVS(avatars)/$image] 0 50"
+	SendToAllUsers "NEW $name [lindex $parms 0] [lindex $parms 1] $image 0 60 [file size $MVS(avatars)/$image] 66 -44"
 	SendToAllUsers "EXIT_OBJ ov_tram_exit [expr $x - 60] [expr $y - 60] [expr $x + 60] [expr $y + 60] 0 [lindex $server_parms 0] $port"
-	after [expr 1000 * $time] "KillOrt $idx"
+	after [expr 500 * $time] "WarnOrt $idx"
 }
 
+# Announce transport departure.
+# 
+# Usage: WarnOrt index
+#
+# This will announce that the transport is getting ready to leave.
+#
+proc WarnOrt {idx} {
+	global MVS
+
+	SendToAllUsers "CHAT $MVS(ORT_info.$idx.name) All Aboard!!!"
+	after [expr 500 * $MVS(ORT_info.$idx.time)] "KillOrt $idx"
+}
+
+# Remove transport.
+#
+# Usage: KillOrt: index
+#
+# Removes a transport telling all user's it's gone.
+#
 proc KillOrt {idx} {
 	global MVS
 
 	set MVS(ORT_current_ort) -1
 	SendToAllUsers "NOMORE $MVS(ORT_info.$idx.name)"
 	SendToAllUsers "EXIT_OBJ ov_tram_exit 0 0 0 0 1 dummyhost 0"
+	#
+	# Clean up the memory this ORT was using.
+	#
+	catch {unset MVS(ORT_info.$idx.host)}
+	catch {unset MVS(ORT_info.$idx.image)}
+	catch {unset MVS(ORT_info.$idx.name)}
+	catch {unset MVS(ORT_info.$idx.port)}
+	catch {unset MVS(ORT_info.$idx.time)}
+	catch {unset MVS(ORT_info.$idx.x)}
+	catch {unset MVS(ORT_info.$idx.y)}
 }
 
 # Disconnect users from the system.
@@ -569,8 +626,6 @@ proc KillOrt {idx} {
 # consuming and an annoucement will be made to all connected users if it
 # is requested with the announce_disconnect parameter (set to 1)
 #
-# Checked for maximum memmory efficiency 12/09/1999 19:20
-#
 proc DisconnectUser {who announce} {
 	global MVS
 	
@@ -594,17 +649,19 @@ proc DisconnectUser {who announce} {
 	#
 	# Clean up the mess this user made!
 	#
-	catch {
-		uplevel #0 unset MVS($who.name) MVS($who.address) MVS($who.port) \
-			MVS($who.ping) MVS($who.ping_response) MVS($who.x) \
-			MVS($who.y) MVS($who.avatar) MVS($who.av_head_x) \
-			MVS($who.av_head_y) MVS($who.av_baloon_x) \
-			MVS($who.av_baloon_y) MVS($who.downloads)
-	}
-	#
-	# TODO (7) Downloads in progress should be canceled!
-	#
-	unset which who announce
+	catch {unset MVS($who.name)}
+	catch {unset MVS($who.downloads)}
+	catch {unset MVS($who.address)}
+	catch {unset MVS($who.av_baloon_x)}
+	catch {unset MVS($who.av_baloon_y)}
+	catch {unset MVS($who.av_head_x)}
+	catch {unset MVS($who.av_head_y)}
+	catch {unset MVS($who.avatar)}
+	catch {unset MVS($who.ping)}
+	catch {unset MVS($who.ping_response)}
+	catch {unset MVS($who.port)}
+	catch {unset MVS($who.x)}
+	catch {unset MVS($who.y)}
 }
 
 # Send Text To a Connect User
@@ -630,8 +687,6 @@ proc SendToUser {who what} {
 # protocol documentation within the technical documentation for a complete
 # descritption of the logic within this function.
 #
-# Checked for maximum memmory efficiency 12/09/1999 22:05
-#
 proc Serv_ProcessInput {who what} {
 	global MVS
 
@@ -947,8 +1002,6 @@ proc Serv_ProcessInput {who what} {
 # This function is used to change a user's avatar. It will announce the
 # change to all connected users.
 #
-# Checked for maximum memory efficiency 12/09/1999 22:14
-#
 proc Serv_ChangeAvatar {who what x y size bx by} {
 	global MVS
 
@@ -992,8 +1045,6 @@ proc Serv_ChangeAvatar {who what x y size bx by} {
 # also checks for the existance of a tickler file which, when it exists,
 # will cause the server to reload it's config file.
 #
-# Checked for maximum memmory efficiency 12/09/1999 23:31
-#
 proc Serv_CheckTimeouts {} {
 	global MVS
 
@@ -1038,9 +1089,15 @@ proc Serv_CheckTimeouts {} {
 		}
 	}
 	#
-	# Check for tickler file
+	# Check for tickler file(s)
 	#
 	if [file exists "$MVS(tickler)"] {ReloadConfig}
+	if [file exists "$MVS(mem_tickler)"] {
+		Serv_DumpMem
+		catch {
+			file delete -force "$MVS(mem_tickler)"
+		}
+	}
 	#
 	# If we're serving, reload this function in 5 seconds.
 	#
@@ -1057,8 +1114,6 @@ proc Serv_CheckTimeouts {} {
 # Will print out information on errors
 # and continue running (we hope)
 #
-# checked for maximum memory efficiency 12/10/1999 08:12
-#
 proc bgerror {stuff} {
 	global errorInfo
 	global errorCode
@@ -1087,8 +1142,6 @@ set MVS(dcc_num) 0
 # This function will check to see if the
 # user is already getting the file named.
 #
-# checked for maximum memmory efficiency 12/10/1999 08:15
-#
 proc MVSerifyAvailable {who what} {
 	global MVS
 
@@ -1110,8 +1163,6 @@ proc MVSerifyAvailable {who what} {
 # avatars and object images. This is a passive dcc transfer so things work
 # better through modern firewalls.
 #
-# Checked for maximum memmory efficiency 12/10/1999 21:38
-#
 proc Serv_DCCSend {who what type} {
 	global MVS
 
@@ -1191,8 +1242,6 @@ proc Serv_DCCSend {who what type} {
 # opening of the server socket. It accepts the connection, closes the
 # server socket and starts the sending of the file to the user.
 #
-# checked for maximum memmory efficiency 12/10/1999 21:40
-#
 proc Serv_acceptSend {index chan hst port} {
 	global MVS
 
@@ -1260,8 +1309,6 @@ proc Serv_acceptSend {index chan hst port} {
 # far. It will end the transfer when the client informs us that it has all
 # of the data we've sent.
 #
-# checked for maximum memmory efficiency 12/10/1999 21:47
-#
 proc Serv_dccSendEvent {index st fd} {
 	global MVS
 	
@@ -1355,8 +1402,6 @@ proc Serv_dccSendEvent {index st fd} {
 #
 # This is our central location for DOWNLOADING files.
 # 
-# checked for maximum memmory efficiency 12/10/1999 22:28
-#
 proc GetBinaryFile {who what size type} {
 	global MVS
 	
@@ -1405,8 +1450,6 @@ proc GetBinaryFile {who what size type} {
 # opening of the server socket. It accepts the connection, closes the
 # server socket and starts the getting of the file to the user.
 #
-# checked for maximum memmory efficiency 12/10/1999 22:35
-#
 proc acceptGet {index chan hst port} {
 	global MVS
 
@@ -1454,8 +1497,6 @@ proc acceptGet {index chan hst port} {
 # far. It will end the transfer when the client informs us that it has all
 # of the data we've sent.
 #
-# checked for maximum memmory efficiency 12/10/1999 21:47
-#
 proc Serv_dccgevent {index st out} {
 	global tl MVS
 
@@ -1526,8 +1567,6 @@ proc Serv_dccgevent {index st out} {
 # avatar to all connected users. Once it's done ending the transfer, it
 # will cleanup the mess made.
 #
-# checked for maximum memmory efficiency 12/10/199 22:58
-#
 proc Serv_endDCC {type index fail_type debug} {
         global MVS tl
 
@@ -1554,15 +1593,23 @@ proc Serv_endDCC {type index fail_type debug} {
 			}
 		}
 	}
-	catch {
-		unset idx tl($MVS(DCC.$index.sock))
-		unset MVS(DCC.$index.time) MVS(DCC.$index.server) \
-		MVS(DCC.$index.file) MVS(DCC.$index.sender) MVS(DCC.$index.size) \
-		MVS(DCC.$index.posn) MVS(DCC.$index.server) MVS(DCC.$index.sock) \
-		MVS(DCC.$index.port) MVS(DCC.$index.remote) \
-		MVS(DCC.$index.av_head_x) MVS(DCC.$index.av_head_y) \
-		MVS(DCC.$index.av_baloon_x) MVS(DCC.$index.av_baloon_y)
-	}
+	#
+	# Clean up the memory this download was using.
+	#
+	catch {unset MVS(DCC.$index.av_baloon_x)}
+	catch {unset MVS(DCC.$index.av_baloon_y)}
+	catch {unset MVS(DCC.$index.av_head_x)}
+	catch {unset MVS(DCC.$index.av_head_y)}
+	catch {unset MVS(DCC.$index.port)}
+	catch {unset MVS(DCC.$index.remote)}
+	catch {unset MVS(DCC.$index.sock)}
+	catch {unset MVS(DCC.$index.type)}
+	catch {unset MVS(DCC.$index.file)}
+	catch {unset MVS(DCC.$index.posn)}
+	catch {unset MVS(DCC.$index.sender)}
+	catch {unset MVS(DCC.$index.server)}
+	catch {unset MVS(DCC.$index.size)}
+	catch {unset MVS(DCC.$index.time)}
 }
 
 #
@@ -1574,8 +1621,6 @@ proc Serv_endDCC {type index fail_type debug} {
 # comare the size against the server limits and report back if the file is
 # good or bad.
 #
-# checked for maximum memmory efficiency 12/10/1999 23:00
-#
 proc CheckGif {file} {
 	global MVS
 
@@ -1608,8 +1653,6 @@ proc CheckGif {file} {
 # This function will send all server objects to a user. It is normally
 # used at connect time to initalize an object for a user. 
 #
-# checked for maximum memmory efficiency 12/10/1999 23:03
-#
 proc SendObjects {who} {
 	global MVS
 	foreach object [glob -nocomplain "$MVS(sobjects)/*"] {
@@ -1739,6 +1782,13 @@ proc DisconnectOrtRegistry {sck} {
 	set idx [lsearch $MVS(registry.servers) $sck]
 	set MVS(registry.servers) [lreplace $MVS(registry.servers) $idx $idx]
 	LogIt "(ORT) Disconnected from Ort Registry"
+	#
+	# Clean up the memory this registry was using.
+	#
+	catch {unset MVS(registry.$sck.password)}
+	catch {unset MVS(registry.$sck.stage)}
+	catch {unset MVS(registry.$sck.timeout)}
+	catch {unset MVS(registry.$sck.username)}
 }
 
 #
@@ -1748,27 +1798,30 @@ proc DisconnectOrtRegistry {sck} {
 #
 proc RegisterWithORTs {} {
 	global MVS
+	
 	set idx 0
-	set MVS(registry.servers) {}
-	foreach server $MVS(ORT_Server) {
-		if {[catch {
-			set parms [split $server ":"]
-			set sck [socket -async [lindex $parms 0] [lindex $parms 1]]
-			fconfigure $sck -blocking 0
-			set MVS(registry.$sck.stage) 0
-			set MVS(registry.$sck.username) [lindex $MVS(ORT_Username) $idx]
-			set MVS(registry.$sck.password) [lindex $MVS(ORT_Password) $idx]
-			puts $sck "TRANS_REG"
-			set MVS(registry.$sck.timeout) [clock seconds]
-			lappend MVS(registry.servers) $sck
-			fileevent $sck readable "ReadFromOrtRegistry $sck"
-			flush $sck
-		} error]} {
-			LogIt "(ERROR) $error"
-		} else {
-			LogIt "(ORT_REG) Registering with $server"
+	if {$MVS(register_ort)} {
+		set MVS(registry.servers) {}
+		foreach server $MVS(ORT_Server) {
+			if {[catch {
+				set parms [split $server ":"]
+				set sck [socket -async [lindex $parms 0] [lindex $parms 1]]
+				fconfigure $sck -blocking 0
+				set MVS(registry.$sck.stage) 0
+				set MVS(registry.$sck.username) [lindex $MVS(ORT_Username) $idx]
+				set MVS(registry.$sck.password) [lindex $MVS(ORT_Password) $idx]
+				puts $sck "TRANS_REG"
+				set MVS(registry.$sck.timeout) [clock seconds]
+				lappend MVS(registry.servers) $sck
+				fileevent $sck readable "ReadFromOrtRegistry $sck"
+				flush $sck
+			} error]} {
+				LogIt "(ERROR) $error"
+			} else {
+				LogIt "(ORT_REG) Registering with $server"
+			}
+			incr idx
 		}
-		incr idx
 	}
 	after 300000 RegisterWithORTs
 	if {$idx} {
@@ -1776,6 +1829,46 @@ proc RegisterWithORTs {} {
 	}
 }
 
+#
+# Serv_DumpMem
+#
+# This function will dump the contents of the MVS array and is mostly used
+# with the TickleMem tickler file to incvoke this function. It allows
+# developers to find and remove memory leaks within the main array. A text
+# file will be created named Dump.mem containing the keys and values of
+# the main array.
+#
+proc Serv_DumpMem {} {
+	global MVS tl
+
+	set arrays [list MVS tl]
+	set outfile [open "$MVS(homedir)/Dump.mem" "w"]
+	#
+	# debug all the arrays.
+	#
+	foreach ar $arrays {
+		puts $outfile "------------------------------------------------------------------------------"
+		puts $outfile " OpenVerse Server - THIS IS THE $ar\() ARRAY"
+		puts $outfile "------------------------------------------------------------------------------"
+		set toggle 0
+		set values {}
+		set keys {}
+		foreach var [array get $ar] {
+			if {!$toggle} {
+				lappend keys $var
+				set toggle 1
+			} else {
+				set toggle 0
+			}
+		}
+		set keys [lsort $keys]
+		foreach key $keys {
+			puts $outfile [format "%-39.39s %-39.39s" $key [set $ar\($key)]]
+		}
+	}
+	close $outfile
+}
+
 #
 # Call our recursing functions.
 # These functions call themselves over time to perform routine repetetive
